from  ..mechanics import *
from sympy.physics.mechanics import dynamicsymbols


class TitlePageComponent(Environment):
    
    latex_name='titlepage'
    
    def __init__(self, system=None, options=None, arguments=None, start_arguments=None,
                 **kwargs):
        r"""
        Args
        ----
        options: str or list or  `~.Options`
            Options to be added to the ``\begin`` command
        arguments: str or list or `~.Arguments`
            Arguments to be added to the ``\begin`` command
        start_arguments: str or list or `~.Arguments`
            Arguments to be added before the options
        """

        self.system = system
        self.options = options
        self.arguments = arguments
        self.start_arguments = start_arguments

        
        
        super().__init__(options=options, arguments=arguments, start_arguments=start_arguments,**kwargs)
        
        if self.system is not None:

        
            system = self.system


            
            self.append(NoEscape('\centering'))

            self.append(NoEscape('\\Huge Mechanical Vibration \n \n'))
            
            self.append(Command('vspace',arguments='1cm'))

            
            
            if len(system.q)==1:
                dof_str = 'Single Degree of Freedom'
            else:
                dof_str = 'Multi Degree of Freedom'
                
            if system._dissipative_potential==0 or system._dissipative_potential is None:
                damping_str = 'UNDAMPED'
            else:
                damping_str = 'DAMPED'

           
            self.append(NoEscape(f'\\Large {dof_str} {damping_str} SYSTEM \n \n'))    
            
            self.append(Command('vspace',arguments='1cm'))
            
            if len(system._given_data) == 0:
                self.append(NoEscape(f'{system.readable_name} with {len(system.q)}DOF \n \n'))
            else:
                data_eqns= [f'${latex(key)} = {latex(value)}$' for key, value in system._given_data.items()]

                self.append(NoEscape(f'''{system.readable_name} with {len(system.q)}DOF for {f', '.join(data_eqns)} \n \n'''))
#                 self.append(NoEscape(','.join(data_eqns))
            
            self.append(Command('vspace',arguments='1cm'))
            
            self.append(Command('MyAuthor'))
            self.append(NoEscape(f'\\par'))
            self.append(Command('vspace',arguments='1cm'))
            
            #self.append(Command('vspace',arguments='1cm'))
            #self.append(NoEscape(f'\\protect\\par'))
            #self.append(NewLine())
            self.append(Command('MyDate'))


            
######################### Initial point
class WUTTitlePage(TitlePageComponent):

    def __init__(self, title=None, large_text=None, graphic_path=None, author=None,
                 system=None, options=None, arguments=None, start_arguments=None,
                 **kwargs):
        r"""
        Args
        ----
        title: str
            Title to be added to the title page
        large_text: str
            Large text to be added to the title page
        graphic_path: str
            Path to the graphic to be included on the title page
        author: str
            Author's name to be added to the title page
        options: str or list or  `~.Options`
            Options to be added to the ``\begin`` command
        arguments: str or list or `~.Arguments`
            Arguments to be added to the ``\begin`` command
        start_arguments: str or list or `~.Arguments`
            Arguments to be added before the options
        """

        self.title = title
        self.large_text = large_text
        self.graphic_path = graphic_path
        self.author = author
        
        super().__init__(system=system, options=options, arguments=arguments, start_arguments=start_arguments, **kwargs)

        if self.title:
            self.append(Command('title', self.title))
        
        if self.large_text:
            self.append(Command('large', self.large_text))
        
        if self.graphic_path:
            self.append(Command('includegraphics', options='[width=5cm,height=5cm]', arguments=self.graphic_path))
        
        if self.author:
            self.append(Command('author', self.author))

    
# Damian
class ExemplaryPictureComponent(ReportComponent):
    title="Example of an real object"
    packages=[Package('float')] 
    @property
    def header_text(self):
        #"Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."
        return "The picture represents a real object."
    @property
    def footer_text(self):


        return "A real example is shown for better understanding of the schematic view and purpose of the system."
    def append_elements(self):
        
        system = self.reported_object

        display(ReportText(self.header_text))
        

        display(Picture(system._real_example(),width='8cm'))


        display(ReportText(self.footer_text))

class ExemplaryPictureDynPyCodeComponent(ReportComponent):

    title="Dynpy Code for Exemplary Picture"

    @property
    def header_text(self):
        return "Code line responsible for extracting exemplary picture from dynamic system is as following:"


    @property
    def footer_text(self):
        return "Picture representing real object of considered dynamic system."

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system

        display(ReportText( self.header_text  ))


        display(ObjectCode(#tu
f'''

from dynpy.models.mechanics import *
from sympy import *
from dynpy.utilities.report import Picture

dyn_sys = {system.__class__.__name__}()

display(Picture(dyn_sys._real_example()))


'''))

        display(ReportText( self.footer_text  ))
        
        
class ExemplaryPictureSymPyCodeComponent(ExemplaryPictureComponent):
    title="SymPy code for calling scheme of the system"

    @property
    def header_text(self):

        return "Code for calling an exemplary picture is as follows:"

    @property
    def footer_text(self):
        
        system = self.reported_object

        return f"Analysis of the real example is needed to propose physical scheme."
    
    
    def append_elements(self):
        
        system = self.reported_object
        path=system._real_example()
        
        display(ReportText(  self.header_text ))
        
        display(ObjectCode(#tu
f'''


    from dynpy.models.mechanics import *
    from sympy import *
    from dynpy.utilities.report import Picture
   
    path="{path}"
    
    display(Picture(path ,width='8cm'))



'''
            ))
        display(ReportText( self.footer_text ))
        
        
# Boogi        
class SchemeComponent(ExemplaryPictureComponent):
    title="Scheme of the system"

    @property
    def header_text(self):
        #"Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."
        return "Scheme of the real object is presented in the figure. It was obtained by the analysis of real mechanism."

        
    @property
    def footer_text(self):
        
        system = self.reported_object
        
        #"Analizując przedstawiony układ można stwierdzić, że jego liczba stopni swobody to {len(system.q)}."
        return f"Analysis of the scheme allows to claim its number of degrees of freedom which is {len(system.q)}"
    
    
    def append_elements(self):
        
        system = self.reported_object

        display(ReportText(  self.header_text ))

        display(Picture(system._scheme(),width='8cm'))
        


        display(ObjectCode( self.footer_text ))


class SchemeDynPyCodeComponent(ExemplaryPictureComponent):
    title="DynPy code for scheme of the system"

    @property
    def header_text(self):

        return "DynPy code for calling the scheme of the dynamic system is as follows:"


    @property
    def footer_text(self):

        system = self.reported_object

        return f"Analysis of the scheme allows to claim its number of degrees of freedom which is {len(system.q)}"

    def append_elements(self):

        system = self.reported_object

        display(ReportText(  self.header_text ))

        display(ObjectCode(#tu

f'''



    from dynpy.models.mechanics import *
    from dynpy.utilities.report import Picture
    from sympy import *

    dyn_sys = {system.__class__.__name__}()


    display(Picture(dyn_sys._scheme(), caption='Scheme of considered dynamic system.'))



'''))

        display(ReportText( self.footer_text ))


class SchemeSymPyCodeComponent(ExemplaryPictureComponent):
    title="SymPy code for scheme of the system"

    @property
    def header_text(self):
        return "SymPy code for calling the scheme of the dynamic system is as following:"

        
    @property
    def footer_text(self):
        
        system = self.reported_object
        return f"Analysis of the scheme allows to claim its number of degrees of freedom which is {len(system.q)}"
    
    
    def append_elements(self):
        
        system = self.reported_object
        path=system._default_folder_path + system.scheme_name

        display(ReportText(  self.header_text ))

        display(ObjectCode(#tu
f'''


    from dynpy.models.mechanics import *
    from dynpy.utilities.report import Picture
    from sympy import *

    path="{path}"

    display(Picture(path ,width='8cm', caption='Scheme of considered system.'))



'''
            ))
        display(ReportText( self.footer_text ))


#TODO
class DamperPlot(ExemplaryPictureComponent):
    title="Siła tłumienia"
  

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys




class NumericalAnalysisComponent(ExemplaryPictureComponent):
    title="Symulacja numeryczna"

    _default_ics = None
    _default_parameters = {}
    _default_tspan = np.linspace(0,1,1000)
    
    @classmethod
    def set_default_ics(cls,default_ics=None):
        
        if default_ics is None:
            cls._default_ics = default_ics
        
        return cls

    @classmethod
    def set_default_parameters(cls,default_parameters=None):
        
        if default_parameters is None:
            cls._default_parameters = default_parameters
        
        return cls
    
    @classmethod
    def set_default_tspan(cls,default_tspan=None):
        
        if default_tspan is None:
            cls._default_tspan = default_tspan
        
        return cls
    

    def append_elements(self):
        
        system = self.reported_object

        display(ReportText(f'''Dla Damiana :P
                            '''))
        if self._default_ics is None:
            ics = list(system.Y*0.0)
        else:
            ics = self._default_ics
            
        sym_res = system.subs(self._default_parameters).numerized().compute_solution(self._default_tspan,ics)

        LatexDataFrame.formatted(sym_res).plotted(preview=True)
        

        display(ReportText(f'''Dla Damiana :P
                            '''))

# Boogi
class KineticEnergyComponent(ReportComponent):
    
    title="Kinetic energy"

    @property
    def header_text(self):
        #"Energia kinetyczna układu wyrażona jest wzorem:"
        return "Kinetic energy of the system has a following form:"

        
    @property
    def footer_text(self):
        #"Wyznaczona wielkość określa energię układu wynikającą z jego własności inercyjnych (energię zmagazynowaną w elementach bezwładnych)."
        return "Determined formula specify energy of the system related to its inertial properties."
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys




        display(ReportText( self.header_text  ))
        

        display(SympyFormula( Eq(Symbol('T'),
                     dyn_sys_lin._kinetic_energy) , marker=None))
        
        
        
               
        display(ReportText( self.footer_text  ))
    
class KineticEnergyDynPyCodeComponent(ReportComponent):

    title="Dynpy Code for Kinetic Energy"

    @property
    def header_text(self):
        return "Code line responsible for extracting kinetic energy from dynamic system is as following:"

    @property
    def footer_text(self):
        return "Determined piece of code specify energy of the system related to its inertial properties."

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys



        display(ReportText( self.header_text  ))

        display(ObjectCode(#tu
f'''

    from dynpy.models.mechanics import *
    from sympy import *

    dyn_sys = {system.__class__.__name__}()

    display( Eq(Symbol('T'),
                     dyn_sys._kinetic_energy) )


'''))



###
        
class KineticEnergySymPyCodeComponent(ReportComponent):

    title="Sympy Code for Kinetic Energy"

    @property
    def header_text(self):
        return "Code line responsible for providing symbolic form of kinetic energy is as follows:"


    @property
    def footer_text(self):
        return "Determined piece of code specify energy of the system related to its inertial properties."

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system


        display(ReportText( self.header_text  ))

#         Ek=dyn_sys._kinetic_energy

        code_list=python(system._kinetic_energy).split('\n')

        var_name = 'Ek'

        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]

        display(ObjectCode(#tu
f'''




    from sympy import *

    {code}


    display(Eq(Symbol('T'),Ek))




'''))

        display(ReportText( self.footer_text  ))

    
class PotentialEnergyComponent(ReportComponent):#Jaś fasola
    
    title="Potential energy"
    @property
    def header_text(self):
        #"Energia potencjalna układu wyrażona jest wzorem:"
        return "Potential energy of the system has a following form:"

        
    @property
    def footer_text(self):
        #"Zaprezentowana zależność opisuje oddziaływanie potencjalnych pól sił w których znajduje się obiekt."
        return "The presented relationship describes the interaction of potential force fields in which the object is located."

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system


        display(ReportText(  self.header_text   ))

        display(SympyFormula( Eq(Symbol('V'),
                     dyn_sys._potential_energy ), marker=None))

        display(ReportText(  self.footer_text   ))

class PotentialEnergyDynPyCodeComponent(ReportComponent):
    
    title="Dynpy Code for Potential Energy"

    @property
    def header_text(self):
        return "Code line responsible for extracting potential energy from dynamic system is as follows:"

        
    @property
    def footer_text(self):
        return "Determined piece of code specify energy of the system, which is stored in object."
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system


        display(ReportText( self.header_text  ))


        display(ObjectCode(# tu 
f'''

    from dynpy.models.mechanics import *
    from sympy import *
    
    dyn_sys = {system.__class__.__name__}()

    display( Eq(Symbol('Ep'),
                     dyn_sys._potential_energy) )

    
'''))

        display(ReportText( self.footer_text  ))
        
        
class PotentialEnergySymPyCodeComponent(ReportComponent):

    title="Sympy Code for Potential Energy"

    @property
    def header_text(self):
        return "Code line responsible for providing symbolic form of potential energy is as following:"


    @property
    def footer_text(self):
        return "Determined piece of code specify energy of the system, which is stored in object."

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system


        display(ReportText( self.header_text  ))

        Ep=dyn_sys._potential_energy

        code_list=python(system._potential_energy).split('\n')

        var_name = 'Ep'

        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]

        display(ObjectCode(#tu
f'''




    from sympy import *

    {code}


    display(Eq(Symbol('E_p'),Ep))




'''))

        display(ReportText( self.footer_text  ))


# Marcel
class DissipationComponent(ReportComponent):
    
    #"Dyssypacyjna funkcja Rayleigh'a"
    title="Dissipative Rayleigh function"
    
    
    @property
    def header_text(self):
        #"Energia rozpraszana tłumieniem wyrażona jest wzorem:"
        return "The energy dissipated by attenuation is given by the formula:"

        
    @property
    def footer_text(self):
        #" Podana zależność stanowi potencjał dysynpacyjny Rayleigh'a, który poddany różniczkowaniu względem wektora prędkości uogólnionych pozwala na określenie sił wiskotycznego tłumienia."
        return "The given dependence is the Rayleigh dissipation potential, which when differentiated against the generalized velocity vector allows to determine the viscous damping forces."
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys

        


        display(ReportText( self.header_text  ))
        

        display(SympyFormula( Eq(Symbol('D'),
             dyn_sys_lin._dissipative_potential) , marker=None ))

        display(ReportText( self.footer_text  ))

# Boogi          
class LagrangianComponent(ReportComponent):
    
    #title="Lagrangian (funkcja Lagrange'a)  układu"
    title="Lagrangian of the system (Lagranges function)"
        
    @property
    def header_text(self):
        #"Lagrangian systemu wyrażony jest wzorem ({AutoMarker(Eq(Symbol('L'),dyn_sys.L.expand()[0]))}):"
        
        system = self.reported_object
        dyn_sys=system
        
        return f"System Lagrangian is described by the formula (marker):"
        
    @property
    def body_text_1(self):
        #"Równania Eulera-Lagrange'a dla rozważanego przypadku są nastęujące: "   
        return "The Euler-Lagrange equations for the case under consideration are as follows: "
        
    @property
    def body_text_2(self):
        #"Kolejne pochodne wynikające z zastosowania równań Eulera-Lagrange'a są nastęujące: "
        return "Subsequent derivatives obtained with the Euler-Lagrange equations are as follows: "
    
    @property
    def footer_text(self):
        #"Wyniki przedstawionych operacji wykorzystuje się wyznaczenia równań ruchu układu."
        return "The results of the presented operations are used to determine the equations of motion of the system."
        
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()

        
        print(system._scheme())
        
        mrk_lagrangian_nonlin=Marker('lagrangianNL',prefix='eq')

        #display(ReportText(f'''The following model is considered. The system's Lagrangian is described by the formula ({Ref(mrk_lagrangian_nonlin).dumps()}):
        #                    '''))
        
        eq_lagr = Eq(Symbol('L'),dyn_sys.L.expand()[0])
        
        display(ReportText(self.header_text.replace('marker', f'{AutoMarker(eq_lagr)}')))


        display((SympyFormula(eq_lagr)))
        
        q_sym =[ Symbol(f'{coord}'[0:-3]) for coord in dyn_sys.q]
        
        diffL_d=lambda coord: Symbol(f'\\frac{{ \\partial L}}{{  \\partial {vlatex(coord)}  }}')
        diffD_d=lambda coord: Symbol(f'\\frac{{ \\partial D}}{{  \\partial {vlatex(diff(coord))}  }}')
        d_dt_diffL_d=lambda coord: Symbol(f'\\frac{{ \\mathrm{{d}}  }}{{  \\mathrm{{d}} {vlatex(system.ivar)}  }} {vlatex(diffL_d(coord))} ')        

        display(ReportText(self.body_text_1))
        
        for coord in dyn_sys.q:
            display((SympyFormula(  Eq(d_dt_diffL_d(coord.diff(system.ivar)) - diffL_d(coord) + diffD_d(coord),Symbol(f'Q_{{ {vlatex(coord)} }}^N'))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
        
        
        display(ReportText(self.body_text_2))
        
        for coord in dyn_sys.Y:
            display((SympyFormula(  Eq(diffL_d(coord),dyn_sys.L.expand()[0].diff(coord))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
            


        for coord in dyn_sys.q.diff(system.ivar):
            display((SympyFormula(  Eq(d_dt_diffL_d(coord),dyn_sys.L.expand()[0].diff(coord).diff(system.ivar))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
        for coord in dyn_sys.q:
            display((SympyFormula(  Eq(diffD_d(coord),(S.One/2 * diff(dyn_sys.q.transpose())* dyn_sys_lin.damping_matrix()* diff(dyn_sys.q))[0].diff(diff(coord)))  , marker=mrk_lagrangian_nonlin,backend=vlatex )  ))
            #display(Markdown(f'\\begin{equation}    \\end{equation}').reported())
        #with doc_model.create(DMath()) as eq:
        #    eq.append(NoEscape(latex(Derivative(Symbol('L'),q_sym[0],evaluate=False))))
        #    eq.append(NoEscape('='))
        #    eq.append(NoEscape(vlatex(dyn_sys.L.expand()[0].diff(dyn_sys.q[0]))))

        mrk_gov_eq_nonlin=Marker('gov_eq_nonlin_sys',prefix='eq')

        #display(ReportText(f'''The governing equations of the system have a following form ({Ref(mrk_gov_eq_nonlin).dumps()}):
        #                    '''))

        display(ReportText(self.footer_text))


        
    def as_frame(self):
        frame=Frame(title=self.title,options=['allowframebreaks'])
        #frame.packages +(self.packages)
        frame+=(list(self))
        return frame

        
    def as_frame(self):
        frame=Frame(title=self.title,options=['allowframebreaks'])
        #frame.packages +(self.packages)
        frame+=(list(self))
        return frame


class LagrangianDynPyCodeComponent(ReportComponent):
    
    title="Dynpy Code for Lagrangian Component"

    @property
    def header_text(self):
        return "Code line responsible for providing symbolic form of Lagrange's function is as following:"

        
    @property
    def footer_text(self):
        return "Considered piece of code can be used to determine the equations of motion of a system."
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system


        display(ReportText( self.header_text  ))


        display(ObjectCode(#tu
f'''



    from dynpy.models.mechanics import *
    from sympy import *

    dyn_sys = {system.__class__.__name__}()
    t = dyn_sys.ivar

    display( Eq(Symbol('L'),
                     dyn_sys.lagrangian()) )

    for q in dyn_sys.q:

        display(dyn_sys.lagrangian().diff(q))

        display(dyn_sys.lagrangian().diff(q.diff(t)).diff(t))



'''))

        display(ReportText( self.footer_text  ))


class LagrangianSymPyCodeComponent(ReportComponent):

    title="Sympy Code for Lagrangian Component"

    @property
    def header_text(self):
        return "Code line responsible for providing symbolic form of Lagrange's function is as following:"

        
    @property
    def footer_text(self):
        return "Considered piece of code can be used to determine the equations of motion of a system."
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system


        display(ReportText( self.header_text  ))


        L=dyn_sys.lagrangian()
        q=dyn_sys.q
        t=dyn_sys.ivar

        code_list=python(system.lagrangian()).split('\n')

        var_name = 'L'

        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]


        display(ObjectCode(#tu
f'''




    from sympy import *

    {code}


    display(Eq(Symbol('L'),L))

    qs = {q}

    for q in qs:

        display(L.diff(q))

        display(L.diff(q.diff(t)).diff(t))



'''))

        display(ReportText( self.footer_text  ))

        
        
# Amadi & Damian
class GoverningEquationComponent(ReportComponent):
    
    #Równania ruchu
    title="Equation of motion"
    
    @property
    def entry_text_one(self):
        system = self.reported_object
        dyn_sys=system
        return f"Using the calculated derivatives, the equation of motion is based on the appropriate formula. System equation of motion is described by the formula ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})"
    
    @property
    def entry_text_two(self):
        system = self.reported_object
        dyn_sys=system
        #f"przedstawiają zależności ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})-({AutoMarker(Eq(dyn_sys._eoms[-1].simplify().expand(),0))})"
        return f"Using the calculated derivatives, the equations of motions are based on the appropriate formulas. System equations of motions are described by the formulas ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})-     ({AutoMarker(Eq(dyn_sys._eoms[-1].simplify().expand(),0))})"

    @property
    def footer_text(self):
        #f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
        return f''' The determined equations constitute a mathematical dynamic description of the properties of the system.
                    Further analysis allows for an effective analysis of the modeled object's operation and determination of its mechanical parameters. '''
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system

        if len(system.q)==1:
            display(ReportText(self.entry_text_one))   
        else:
            display(ReportText(self.entry_text_two))   

        
        for eq in dyn_sys._eoms:
            display(SympyFormula( Eq(eq.simplify().expand(),0) , marker=None))

        display(ReportText(self.footer_text))   

        
class LinearizedGoverningEquationComponent(ReportComponent):
    #Równania ruchu
    title="Equation of motion"
    
    @property
    def entry_text_one(self):
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()
        #f"przedstawia zależność ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})"
        return f"Using the calculated derivatives, the equation of motion is based on the appropriate formula. Linearized system equation of motion is described by the formula ({AutoMarker(Eq(dyn_sys_lin._eoms[0].simplify().expand(),0))})"
    
    @property
    def entry_text_two(self):
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()
        #f"przedstawiają zależności ({AutoMarker(Eq(dyn_sys._eoms[0].simplify().expand(),0))})-({AutoMarker(Eq(dyn_sys._eoms[-1].simplify().expand(),0))})"
        return f"Using the calculated derivatives, the equations of motions are based on the appropriate formulas. Linearized system equations of motions are described by the formulas ({AutoMarker(Eq(dyn_sys_lin._eoms[0].simplify().expand(),0))})-({AutoMarker(Eq(dyn_sys_lin._eoms[-1].simplify().expand(),0))})"
    
    @property
    def footer_text(self):
        #f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
        return f''' The determined equations constitute a mathematical dynamic description of the properties of the system.
                    Further analysis allows for an effective analysis of the modeled object's operation and determination of its mechanical parameters. '''
    
    def append_elements(self):
        
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys.linearized()


        
        if len(system.q)==1:
            display(ReportText(self.entry_text_one))   
        else:
            display(ReportText(self.entry_text_two))   
        
        for eq in dyn_sys._eoms:
            display(SympyFormula( Eq(eq.simplify().expand(),0) , marker=None))

        display(ReportText(self.footer_text))   
        
        
class GoverningEquationDynpyCodeComponent(ReportComponent):
    
    #Równania ruchu
    title="Equation of motion"
    
    @property
    def header_text(self):
      
      return 'Presented piece of code describes equations of motoion of the system distributed from dynamical system'

       

    @property
    def footer_text(self):
        #f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
        return 'Execution of this code returns equations of motion generated from dynamic system'
    
    def append_elements(self):
        system = self.reported_object
        dyn_sys=system
        eoms=str(dyn_sys._eoms)
        
        display(ReportText(self.header_text))

        display(ObjectCode(#tu
f'''
    
    


    from dynpy.models.mechanics import *
    from sympy import *
    
    dyn_sys = {system.__class__.__name__}()
    
    
    display(Eq(dyn_sys._eoms,0,evaluate=False))



    
'''))

        display(ReportText(self.footer_text))        
        
class GoverningEquationSympyCodeComponent(ReportComponent):
    
    #Równania ruchu
    title="Equation of motion"
    
    @property
    def header_text(self):
      
      return 'Presented piece of code describes equations of motoion of the system created using Sympy library'

       

    @property
    def footer_text(self):
        #f''' Wyznaczone równania stanowią matematyczny opis dynamiczny właściwości układu. Dalsza analiza pozwala na skuteczną analizę działania modelowanego obiektu i określenie jego parametrów mechanicznych. '''
        return 'Execution of this code returns equations of motion in a form of Sympy outputs'
    
    def append_elements(self):
        system = self.reported_object
        dyn_sys=system
        eoms=str(dyn_sys._eoms)
        display(ReportText(self.header_text))
        
        code_list=python(system._eoms).split('\n')
        
        var_name = 'eoms'
        
        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]

        

        display(ObjectCode(#tu
f'''
    
    


    from dynpy.models.mechanics import *
    from sympy import *
    
    {code} 
    
    display(Eq(eoms,0,evaluate=False))

    
'''))

        display(ReportText(self.footer_text))

class LinearizationComponent(ReportComponent): # Szymon

    title="Linearization of equation of motion"

    @property
    def entry_text(self):
        return "Linearization of governing equations is about finding Taylor series with respect to generalized coordinates, velocities and accelerations in the neighbourhood of the equilibrium point. Following symbols have been introduced to make a simplification."

    @property
    def equilibrium_point_text(self):
        return "Equilibrium points of the system have following forms:"

    @property
    def eom_text(self):
        dyn_sys= self.reported_object
        return '''Equation of motion for coordinate ${coord}$ can be presented as:'''

    @property
    def lagrange_text(self):
        dyn_sys= self.reported_object
        return "Proper computings requires finding derivatives of generalized coordinates, which are components of Lagrange's equations"

    @property
    def derivative_text(self):
        return "The calculated derivatives have a following form:"

    @property
    def linearized_eq_text(self):
        return "The following equation (linearized) can be obtained after substitution of calculated derivatives."

    def append_elements(self):

        system = self.reported_object
        ReportText.set_directory('./SDAresults')
        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        mrk_lagrangian_nonlin = Marker('lagrangLin',prefix='eq')
        mrk_lagrangian_lin = Marker('lagrangLin',prefix='eq')

        t=system.ivar

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()
        coords=tuple(list(dyn_sys.Y) + list(dyn_sys.q.diff(t,t)))
        op_point = {coord: 0 for coord in coords}
        op_point.update(dyn_sys._op_points(subs=True)[0])

        display(ReportText(self.entry_text))


        for coord in coords:
            display((SympyFormula(  Eq(Symbol(vlatex(coord)),Symbol(latex(coord))) , marker=None  )  )) 



        display(ReportText( self.equilibrium_point_text ))

        for eq_coord,val in op_point.items():
            display((SympyFormula(  Eq(eq_coord,val) , marker=mrk_lagrangian_lin  )  ))


        diffL_d=lambda coord: Symbol(latex(Derivative(Symbol('L'),Symbol(vlatex(coord))))  )



        for no,eom in enumerate(dyn_sys._eoms):




            eq_sym=Symbol(f'RR_{latex(dyn_sys.q[no])}')


            display(ReportText( self.eom_text.format(coord = latex(dyn_sys.q[no])) ))

            #display((SympyFormula(  Eq(eq_sym,eom,evaluate=False) , marker=mrk_lagrangian_lin,backend=latex  )  ))


            display(ReportText( self.lagrange_text ))


            display((SympyFormula(  Eq(MultivariableTaylorSeries(eq_sym,coords,n=1,x0=op_point)._symbolic_sum(),0) , marker=None,backend=latex  )  ))

            diff_list=MultivariableTaylorSeries(eom,coords,n=1,x0=op_point).calculation_steps(expr_symbol=eq_sym)

            display(ReportText( self.derivative_text ))

            for diff_eq in diff_list:

                display((SympyFormula(  diff_eq , marker=mrk_lagrangian_lin,backend=latex  )  ))

            display(ReportText( self.linearized_eq_text ))
            display((SympyFormula(  Eq(MultivariableTaylorSeries(eom,coords,n=1,x0=op_point).doit().expand().simplify().expand(),0,evaluate=False) , marker=mrk_lagrangian_lin,backend=latex  )  ))



        AutoBreak.latex_backend = latex_store

class CriticalPointsComponent(ReportComponent):

    @property
    def header_text(self):
        return ('From the relvant kinematic dependencies the critical points of the system are calculated')
    
    @property
    def footer_text(self):
        return('Resultant values of generalized coordinates provide static equilibrium of the system')
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys = system

        display(ReportText(self.header_text))

        for point in dyn_sys._op_points():
          display([Eq(key,Val) for key, Val in point.items()])

        display(ReportText(self.footer_text))

class CriticalPointsDynPyCodeComponent(ReportComponent):

    @property
    def header_text(self):
        return ('The following piece of code allows calculation of critical points for the given system')
    
    @property
    def footer_text(self):
        return('Outputs of this code can be used for further calculation of non-linear systems')
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys = system

        display(ReportText(self.header_text))
        display(ReportText(
f'''
    
    from dynpy.models.mechanics import *
    from sympy import *
    
    dyn_sys = {system.__class__.__name__}()
    
    display(dyn_sys._op_points())
    
'''))
       
        display(ReportText(self.footer_text))

class CriticalPointsSymPyCodeComponent(ReportComponent):

    @property
    def header_text(self):
        return ('From the relvant kinematic dependencies the critical points of the system are calculated')
    
    @property
    def footer_text(self):
        return('Resultant values of generalized coordinates provide static equilibrium of the system')
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system

        critical_points=str(dyn_sys._op_points())

        display(ReportText(self.header_text))

        code_list=python(system._op_points()).split('\n')
        
        var_name = 'critical_points'
        
        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]

        display(ReportText(
f'''

    from dynpy.models.mechanics import *
    from sympy import *
    
    {code} 
    
    display(critical_points)

'''))

        display(ReportText(self.footer_text))

# Marcel & Monika
class FundamentalMatrixComponent(ReportComponent):

    #title="Wyznaczanie macierzy fundamentalnej"
    title="Determining fundamental matrix";

    @property
    def header_text(self):
        #"Z równań ruchu wyznaczono macierz mas i sztywności układu:"
        # google tlumacz
        return "The matrix of masses and stiffnesses of the system was determined from the equations of motion:"

    @property
    def body_text(self):
        #"Macierz fundamentalna, na podstawie której wyznaczono równanie charakterystyczne rozważanego układu #${latex(Delta)}$, przedstawiają się następująco:"
        # google tlumacz
        return "The fundamental matrix, on the basis of which the characteristic equation of the considered system ${delta}$ was determined, is as follows:"

    @property
    def footer_text(self):
        #" Macierz fundamentalna pozwala określić rozwiązanie ustalone. Natomiast bazując na równaniu #charakterystycznym określa się częstości własne układu."
        # google tlumacz
        return "The fundamental matrix allows you to define a fixed solution. On the other hand, based on the characteristic equation, the eigenfrequencies of the system are determined."

    def append_elements(self):

        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store

        t=system.ivar

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()#.subs(system._given_data)


        display(ReportText(self.header_text))

        display((SympyFormula(  Eq(Symbol('M'),dyn_sys_lin.inertia_matrix(),evaluate=False) , marker='a' )  ))

        display((SympyFormula(  Eq(Symbol('K'),dyn_sys_lin.stiffness_matrix(),evaluate=False) , marker='a')  ))

        Delta = Symbol('\Delta')

        display(ReportText(self.body_text.format(delta = latex(Delta) )))

        display((SympyFormula(  Eq(Symbol('A'),dyn_sys_lin.fundamental_matrix(),evaluate=False) , marker='a'  )  ))
        display((SympyFormula(  Eq(Delta,dyn_sys_lin.fundamental_matrix().det().expand().simplify().simplify().expand(),evaluate=False) , marker='a',backend=latex  )  ))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store

class FundamentalMatrixDynPyCodeComponent(ReportComponent):
    
    #title="Wyznaczanie macierzy fundamentalnej"
    title="Determining fundemental matrix component";
    
    @property
    def header_text(self):
        #"Z równań ruchu wyznaczono macierz mas i sztywności układu:"
        # google tlumacz
        return "The matrix of masses and stiffnesses of the system was determined from the equations of motion:"

        
    @property
    def body_text(self):
        #"Macierz fundamentalna, na podstawie której wyznaczono równanie charakterystyczne rozważanego układu ${latex(Delta)}$, przedstawiają się następująco:"
        # google tlumacz
        return "The fundamental matrix, on the basis of which the characteristic equation of the considered system ${latex (Delta)}$ was determined, is as follows:"
    
    @property
    def footer_text(self):
        #" Macierz fundamentalna pozwala określić rozwiązanie ustalone. Natomiast bazując na równaniu charakterystycznym określa się częstości własne układu."
        # google tlumacz
        return "The fundamental matrix allows you to define a fixed solution. On the other hand, based on the characteristic equation, the eigenfrequencies of the system are determined."
    
    
    def append_elements(self):
        
        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()


        display(ReportText(self.header_text))
        
        display((SympyFormula(  Eq(Symbol('M'),dyn_sys_lin.inertia_matrix(),evaluate=False) , marker='a' )  ))

        display((SympyFormula(  Eq(Symbol('K'),dyn_sys_lin.stiffness_matrix(),evaluate=False) , marker='a')  ))

        Delta = Symbol('\Delta')

        display(ReportText(self.body_text))

        display((SympyFormula(  Eq(Symbol('A'),dyn_sys_lin.fundamental_matrix(),evaluate=False) , marker='a'  )  ))
        display((SympyFormula(  Eq(Delta,dyn_sys_lin.fundamental_matrix().det().expand().simplify().simplify().expand(),evaluate=False) , marker='a',backend=latex  )  ))
        
        display(ReportText('''
        display((SympyFormula(  Eq(Symbol('M'),dyn_sys_lin.inertia_matrix(),evaluate=False) , marker='a' )  ))

        display((SympyFormula(  Eq(Symbol('K'),dyn_sys_lin.stiffness_matrix(),evaluate=False) , marker='a')  ))

        Delta = Symbol('\Delta')
        
        display((SympyFormula(  Eq(Symbol('A'),dyn_sys_lin.fundamental_matrix(),evaluate=False) , marker='a'  )  ))
        display((SympyFormula(  Eq(Delta,dyn_sys_lin.fundamental_matrix().det().expand().simplify().simplify().expand(),evaluate=False) , marker='a',backend=latex  )  ))
        
        
        '''))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store    

class FundamentalMatrixSymPyCodeComponent(ReportComponent):
    
    title="Determining fundemental matrix component";
    
    @property
    def header_text(self):
        return "The mass and stiffness matrices of the system were determined from the equations of motion as follows:"

    @property
    def body_text(self):

        return "The fundamental matrix, based on which the characteristic equation of the considered system ${latex(Delta)}$ was determined, is represented as follows:"
    
    @property
    def footer_text(self):
        return "The fundamental matrix allows for determining the steady-state solution. Meanwhile, based on the characteristic equation, one often determines the eigenfrequencies of the system."
    
    def append_elements(self):
        
        system = self.reported_object

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()

        display(ReportText(self.header_text))
        
        
        display((SympyFormula(  Eq(Symbol('M'),dyn_sys_lin.inertia_matrix(),evaluate=False))))

        display((SympyFormula(  Eq(Symbol('K'),dyn_sys_lin.stiffness_matrix(),evaluate=False))))

        Delta = Symbol('\Delta')

        display(ReportText(self.body_text))

        display((SympyFormula(  Eq(Symbol('A'),dyn_sys_lin.fundamental_matrix(),evaluate=False))))
        display((SympyFormula(  Eq(Delta,dyn_sys_lin.fundamental_matrix().det().expand().simplify().simplify().expand(),evaluate=False))))
        
        
        code_list=python(system.fundamental_matrix()).split('\n')

        var_name = 'Fund_mat'

        code = '\n\n\t'.join(code_list[:-1]) + '\n\n\t' + var_name +code_list[-1][1:]

        display(ObjectCode(#tu
f'''

    from sympy import *

    {code}


    display(Eq(Symbol('Delta'),Fund_mat))

'''))
        
        display(ReportText(self.footer_text))
    
# Mateusz
class GeneralSolutionComponent(ReportComponent):
    #"Rozwiązanie ogólne"
    title="General solution"
    
    @property
    def header_text(self):
        #'Rozwiązanie ogólne przedstawia wyrażenie:'
        return "General solution is presented by expression:"
    @property
    def footer_text(self):
        #'Rozwiązanie ogólne opisuje ruch analizowanego układu (przedstawia przemieszczenie w funkcji czasu) i wynika z rozważań dotyczących drgań swobodnych układu.'
        return "General solution describes motion of the analised system - presents displacement i function of time - and is given by considerations about free vibrations of the system"
    
    def append_elements(self):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        #from dynpy import LagrangesDynamicSystem, HarmonicOscillator

        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        
        
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()


        display(ReportText(self.header_text))
#         display((SympyFormula(  Eq(Symbol('X'),HarmonicOscillator(dyn_sys_lin).general_solution()[0].n(3),
#                                             evaluate=False) , marker='a',backend=latex  )  ))
#         display(dyn_sys_lin._eoms)
#         display((SympyFormula(  Eq(Symbol('X_g'), dyn_sys_lin._ode_system.general_solution.n(3), evaluate=False) , marker='a',backend=latex  )  ))

        for i,qs in enumerate(system.q):

            display((SympyFormula(  Eq(Symbol(f'X_g_-{qs}'),dyn_sys_lin._ode_system.general_solution.rhs.n(3)[i], evaluate=False) , marker='a',backend=latex  )  ))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store
        
class GeneralSolutionDynpyCodeComponent(ReportComponent):
    #"Rozwiązanie ogólne"
    title="Dynpy code for general solution"
    
    @property
    def header_text(self):
        #'Rozwiązanie ogólne przedstawia wyrażenie:'
        return "Dynpy code for general solution is presented by expression:"
    @property
    def footer_text(self):
        #'Rozwiązanie ogólne opisuje ruch analizowanego układu (przedstawia przemieszczenie w funkcji czasu) i wynika z rozważań dotyczących drgań swobodnych układu.'
        return "General solution describes motion of the analised system - presents displacement i function of time - and is given by considerations about free vibrations of the system. This code presents a solution in dynpy code."
    
    def append_elements(self):
        system=self.reported_object
        t=system.ivar
        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()
        display(ReportText(self.header_text))
        coord=dyn_sys.q
        
        ode=system._ode_system

        display(ReportText(
f'''

    from dynpy.models.mechanics import *
    from sympy import *

    dyn_sys = {system.__class__.__name__}()
    
    
    dyn_sys._ode_system.general_solution
    


'''))
        
        display(ReportText(self.footer_text))
        
class GeneralSolutionSympyCodeComponent(ReportComponent):
    #"Rozwiązanie ogólne"
    title="Sympy code for General Solution"
    
    @property
    def header_text(self):
        #'Rozwiązanie ogólne przedstawia wyrażenie:'
        return "Sympy code for General Solution is presented by expression:"
    @property
    def footer_text(self):
        #'Rozwiązanie ogólne opisuje ruch analizowanego układu (przedstawia przemieszczenie w funkcji czasu) i wynika z rozważań dotyczących drgań swobodnych układu.'
        return "General solution describes motion of the analised system - presents displacement i function of time - and is given by considerations about free vibrations of the system.  This code presents a solution in sympy code."
    
    def append_elements(self):
        system=self.reported_object
        t=system.ivar
        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()
        ode=system._ode_system.lhs-system._ode_system.rhs
        cord=system.q

        
      
        display(ReportText(self.header_text))

        code_list_ode=python(ode).split('\n')

        var_name = 'ode'
        
        code_ode = '\n\n\t'.join(code_list_ode[:-1]) + '\n\n\t' + var_name +code_list_ode[-1][1:]

        display(ReportText(self.footer_text))

        code_list_dvars=python(system.q).split('\n')

        var_name = 'dvars'
        
        code_dvars = '\n\n\t'.join(code_list_dvars[:-1]) + '\n\n\t' + var_name +code_list_dvars[-1][1:]
        
        
        display(ReportText(
f'''
    from sympy import *
    init_printing()
    

    {code_ode}
    
    {code_dvars}
    
    dsolve(ode,dvars)


'''))

        
# Grześ
class FrequencyResponseFunctionComponent(ReportComponent):
    #"Charakterystyka Amplitudowo-Częstotliwościowa"
    title="amplitude-frequency characteristic"
    
    @property
    def header_text(self):
        #"funkcja odpowiedzi częstotliwościowej:"
        return "Frequency response is given by the formula:"
    @property
    def footer_text(self):
        #"jest to suma kwadratów amplitud pod pierwiastkiem:"
        return "it is the sum of the squared amplitudes under the root:"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('frf'),
                     dyn_sys.frequency_response_function() ), marker=None))

        display(ReportText(self.footer_text))
        
FRFComponent = FrequencyResponseFunctionComponent


#Pati
class FrequencyResponseFunctionComponentToSecond(ReportComponent):
    
    title="sens mocy układu"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys

    @property
    def header_text(self):
        #"sens mocy układu:"
        return "system power sense:"

        display(SympyFormula( Eq(Symbol('frf^2'),
                     dyn_sys.frequency_response_function().doit()**2 ), marker=None))
    @property
    def footer_text(self):
        #"Charakterystyka Amplitudowo-Częstotliwościowa podniesiona do kwadratu"
        return "The amplitude-frequency response squared"
#Pati        
class SpringForce(ReportComponent):
    
    title="Spring force"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


    @property
    def header_text(self): 
        #"Siła od sprężyny wyrażona jest wzorem:""
        return "The spring force is given by the formula:"

    @property
    def middle_text(self): 
        #"zamiast x używam steady solution"
        return "steady solution was used instead of z"

    @property
    def footer_text(self):
           #"Siła od sprężyny, zwana również siłą naciągu, pojawia sie przy ściskaniu lub rozciaganiu. Siła, która działa jest przeciwnie skierowana do ruch i chce przywrócić do pierwotnego jej położenia. Zależy od sztywności sprężyny k oraz od tego o ile została rozciagnieta bądź skrócona x."
        return "Spring force, also known as pull force, occurs when compressed or stretched. The force that acts is opposite to the movement and wants to return it to its original position. It depends on the spring stiffness k and how much it is stretched or shortened z."

    def append_elements(self):


        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F'),-1*system._left_mount.stiffness*system.z,evaluate=False)))

        display(ReportText(self.middle_text))
        
        display(SympyFormula( Eq(Symbol('F'),-1*system._left_mount.stiffness*system._ode_system.steady_solution.rhs[0],evaluate=False)))
        
        display(ReportText(self.footer_text))
        
FRFComponent = FrequencyResponseFunctionComponent
#Pati
class DamperForce(ReportComponent):
    
    title="Siła tłumienia"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


    @property
    def header_text(self): 
        #"Siła tłumienia wyrażona jest wzorem:"
        return "The damping force is given by the formula:"

        display(SympyFormula( Eq(Symbol('F'),-1*system.c*system.v,evaluate=False)))

    @property
    def header_text(self): 
             #"zastępuje prędkość jako pochodna steady stolution po czasie:"
        return "replaces velocity as a derivative of the steady solution over time:"

        display(SympyFormula( Eq(Symbol('F'),-1*system.c*system._ode_system.steady_solution().rhs.diff(system.ivar),evaluate=False)))

    @property
    def footer_text(self):
         #"Siła tłumienia zmniejsza amplitude drgań, ma zwrot przeciwny do prędkości. Zależy od współczynnika tłumienia b oraz od prędkości v."
        return "Siła tłumienia zmniejsza amplitude drgań, ma zwrot przeciwny do prędkości. Zależy od współczynnika tłumienia b oraz od prędkości v."
#Pati
class LogarithmicDecrement(ReportComponent):
    
    title="logarytmiczny dekrement tłumienia liczony z amplitud"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


    @property
    def header_text(self): 
        #"Logarytmiczny dekrement tłumienia wyrażona jest wzorem przy pomocy amplitud:"
        return "The logarithmic decrement of damping is given by the formula in terms of amplitudes."

        display(SympyFormula( Eq(Symbol('delta'),log(system.A_n1/system.A_n2))))

    @property
    def footer_text(self): 
            #"Zaprezentowana zależność opisuje bezwymaiarową wielkość charakteryzującą intensywność tłumienia drgań swobodnych w przypadku podkrytycznym. Jest to wielkość stała dla rozpaywanego                                  układu drgającego i nie zależy od warrunków początkowych. Zależy natomiast od maksymalnych wychyleń w chwilach różniących się o okres drań tłumionych."
            return "The presented dependence describes a dimensionless quantity characterizing the intensity of damping of free vibrations in the subcritical case. It is a constant value for the considered oscillating system and does not depend on the initial conditions. However, it depends on the maximum deflections at moments differing by the period of damped bastards."
    @property
    def header_text(self): 
           #"Logarytmiczny dekrement tłumienia u wyrażona jest wzorem przy pomocy okresu drgań tłumionych:"
        return "The logarithmic damping decrement u is given by the formula using the period of damped vibrations:"

        display(SympyFormula( Eq(Symbol('delta'),2*pi*system.damping_coefficient()*(system.natural_frequencies())**-1,evaluate=False)))
        
    @property
    def footer_text(self): 
           #"Zaprezentowana zależność opisuje bezwymaiarową wielkość charakteryzującą intensywność tłumienia drgań swobodnych w przypadku podkrytycznym. Jest to wielkość stała dla rozpaywanego układu drgającego i nie zależy od warrunków początkowych.Zależy natomiast od okresu drań tłumionych i współczynnika h."
        return "The presented dependence describes a dimensionless quantity characterizing the intensity of damping of free vibrations in the subcritical case. It is a constant value for the considered vibrating system and does not depend on the initial conditions, but it depends on the period of damped vibrations and the coefficient h."



        
# Marcel & Monika
class SteadySolutionComponent(ReportComponent):
    
        #"Rozwiązanie szczególne"
    title="Steady solution"
    _phi=False
    
    @property
    def header_text(self):
        #"Rozwiązanie szczególne przedstawia wyrażenie:"
        # google tlumacz
        return "The steady solution is given by the formula:"

        
    @property
    def footer_text(self):
        #" Rozwiązanie szczególne związane jest obecnością wielkości wymuszających ruch (drgania) analizowanego układu."
        # google tlumacz
        return "The specific solution is related to the presence of quantities that force motion (vibrations) of the analyzed system."
    
    def append_elements(self,phi=_phi):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        
        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()



        display(ReportText(self.header_text ))

#         display((SympyFormula(  Eq(Symbol('X_s'),
#                             HarmonicOscillator(dyn_sys_lin).steady_solution()[0].n(3),
#                             evaluate=False) , marker='b',backend=latex  )  ))
        
#         display((SympyFormula(  Eq(Symbol('X_s'), dyn_sys_lin._ode_system.steady_solution.n(3), evaluate=False) , marker='b',backend=latex  )  ))
        for i,qs in enumerate(system.q):

            display((SympyFormula(  Eq(Symbol(f'X_s_-{qs}'),dyn_sys_lin._ode_system.steady_solution.rhs.n(3)[i], evaluate=False) , marker='b',backend=latex  )  ))

        AutoBreak.latex_backend = latex_store

        display(ReportText(self.footer_text ))
        
#### Amadi
class MaxStaticForce(ReportComponent):
    
    #Maksymalna siła statyczna
    title="Maximum static force"

    @property
    def header_text(self):
        #"Wartość maksymalna siły statycznej działającej na pojedynczy element mocujący:"
        return "Maximum value of the static force acting on a single element:"
    
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_s'),
                     dyn_sys.max_static_force().doit()/2 ), marker=None))
        
#### Amadi
class MaxDynamicForce(ReportComponent):
    
    #Maksymalna siła statyczna
    title="Maximum dynamic force"

    @property
    def header_text(self):
        #"Wartość maksymalna siły dynamicznej działającej na pojedynczy element mocujący:"
        return "Maximum value of the dynamic force acting on a single element:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_d'),
                     dyn_sys.max_dynamic_force().doit() ), marker=None))
        
        
#### Amadi
class DynamicPinDiameter(ReportComponent):
    
    title="Minimum diameter of pin due to dynamic force"

    @property
    def header_text(self):
        return "Minimum diameter of the pin due to dynamic force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('d'),
                     dyn_sys.dynamic_force_pin_diameter().doit() ), marker=None))
        
#### Amadi
class StaticPinDiameter(ReportComponent):
    
    title="Minimum diameter of pin due to static force"

    @property
    def header_text(self):
        return "Minimum diameter of the pin due to static force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('d'),
                     dyn_sys.static_force_pin_diameter().doit() ), marker=None))
        
#### Amadi
class TensionerForce(ReportComponent):
    
    title="Tensioner force"


    @property
    def header_text(self): 
        #"Siła od sprężyny wyrażona jest wzorem:""
        return "The tensioner force modelled as a spring is given by the formula:"

    @property
    def middle_text(self): 
        #"zamiast x używam steady solution"
        return "steady solution was used instead of z"

    @property
    def footer_text(self):
           #"Siła od sprężyny, zwana również siłą naciągu, pojawia sie przy ściskaniu lub rozciaganiu. Siła, która działa jest przeciwnie skierowana do ruch i chce przywrócić do pierwotnego jej położenia. Zależy od sztywności sprężyny k oraz od tego o ile została rozciagnieta bądź skrócona x."
        return "Spring force, also known as pull force, occurs when compressed or stretched. The force that acts is opposite to the movement and wants to return it to its original position. It depends on the spring stiffness k and how much it is stretched or shortened z."

    def append_elements(self):


        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F'),-1*system._tensioner.stiffness*(system.z0-system.z),evaluate=False)))

        display(ReportText(self.middle_text))
        
        display(SympyFormula( Eq(Symbol('F'),-1*system._tensioner.stiffness*system.steady_solution().rhs[0],evaluate=False)))
        
        display(ReportText(self.footer_text))
        
#### Amadi
class TensionerDamperForce(ReportComponent):
    
    title="Tensioner damping force"


    @property
    def header_text(self): 
        #"Siła tłumienia wyrażona jest wzorem:"
        return "The damping force is given by the formula:"


    @property
    def middle_text(self): 
             #"zastępuje prędkość jako pochodna steady stolution po czasie:"
        return "replaces velocity as a derivative of the steady solution over time:"


    @property
    def footer_text(self):
         #"Siła tłumienia zmniejsza amplitude drgań, ma zwrot przeciwny do prędkości. Zależy od współczynnika tłumienia b oraz od prędkości v."
        return "The damping force, which reduces the amplitude of vibrations, has a direction opposite to the velocity. It depends on the damping coefficient c and the speed V."

    def append_elements(self):


        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F'),-1*system._tensioner_damping.c*Symbol('V'),evaluate=False)))

        display(ReportText(self.middle_text))
        
        display(SympyFormula( Eq(Symbol('F'),-1*system._tensioner_damping.c*system.steady_solution().rhs[0].diff(system.ivar),evaluate=False)))
        
        display(ReportText(self.footer_text))
        

#### Amadi
class StaticKeyLength(ReportComponent):
    
    title="Minimum length of key due to static force"

    @property
    def header_text(self):
        return "Minimum length of the key due to static force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('l'),
                     dyn_sys.static_key_length().doit() ), marker=None))
        
#### Amadi
class DynamicKeyLength(ReportComponent):
    
    title="Minimum length of key due to dynamic force"

    @property
    def header_text(self):
        return "Minimum length of the key due to dynamic force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('l'),
                     dyn_sys.dynamic_key_length().doit() ), marker=None))

class StaticAndDynamicCableForceComponent(ReportComponent):
    
    title="Pendulum's cable longitudinal force value"

    @property
    def header_text(self):
        return "Pendulum's cable static force is represented by formula:"
    
    @property
    def footer_text(self):
        return "Pendulum's cable dynamic force is represented by formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys

        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_s'),
                     dyn_sys.max_static_cable_force().doit() ), marker=None))
        
        display(ReportText(self.footer_text))

        display(SympyFormula( Eq(Symbol('T'),
                     dyn_sys.max_dynamic_cable_force().doit() ), marker=None))

        
class PendulumLongitudinalForce(StaticAndDynamicCableForceComponent):
    
    @property
    def middle_text(self):
        return "Pendulum's maximum angular velocity has following form:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.middle_text))

        display(SympyFormula( Eq(Eq(dynamicsymbols('varphi_max').diff(dyn_sys.ivar) , Symbol('A_omega'), evaluate=False),
                     dyn_sys.linearized().frequency_response_function() * dyn_sys.Omega, evaluate=False ), marker=None))
        

class MDoFGeneralSolutionComponent(ReportComponent):
    #"Rozwiązanie ogólne"
    title="General solution"
    
    @property
    def header_text(self):
        #'Rozwiązanie ogólne przedstawia wyrażenie:'
        return "General solution is presented by expression:"
    @property
    def footer_text(self):
        #'Rozwiązanie ogólne opisuje ruch analizowanego układu (przedstawia przemieszczenie w funkcji czasu) i wynika z rozważań dotyczących drgań swobodnych układu.'
        return "General solution describes motion of the analised system - presents displacement i function of time - and is given by considerations about free vibrations of the system"
    
    def append_elements(self):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        #from dynpy import LagrangesDynamicSystem, HarmonicOscillator

        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        
        
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()


        display(ReportText(self.header_text))

        for gen_sol in HarmonicOscillator(dyn_sys_lin.linearized()).general_solution().n(3):
                 display((SympyFormula(Eq(Symbol('X'), gen_sol, evaluate=False) , marker='a',backend=latex)))

        display(ReportText(self.footer_text))

        AutoBreak.latex_backend = latex_store

class MDoFSteadySolutionComponent(ReportComponent):
    
        #"Rozwiązanie szczególne"
    title="Steady solution"
    _phi=False
    
    @property
    def header_text(self):
        #"Rozwiązanie szczególne przedstawia wyrażenie:"
        # google tlumacz
        return "The steady solution is given by the formula:"

        
    @property
    def footer_text(self):
        #" Rozwiązanie szczególne związane jest obecnością wielkości wymuszających ruch (drgania) analizowanego układu."
        # google tlumacz
        return "The specific solution is related to the presence of quantities that force motion (vibrations) of the analyzed system."
    
    def append_elements(self,phi=_phi):

        from ....dynamics import LagrangesDynamicSystem, HarmonicOscillator
        
        system = self.reported_object
        ReportText.set_directory('./SDAresults')

        latex_store=AutoBreak.latex_backend
        AutoBreak.latex_backend = latex_store
        
        t=system.ivar
        

        dyn_sys=system
        dyn_sys_lin=dyn_sys.linearized()



        display(ReportText(self.header_text ))

        steady_sol=dyn_sys_lin._fodes_system.steady_solution

        for q in dyn_sys_lin.q:

            display((SympyFormula(  Eq(Symbol('X_s'), steady_sol.as_dict()[q], evaluate=False) , marker='b',backend=latex)))

        AutoBreak.latex_backend = latex_store

        display(ReportText(self.footer_text ))
        
        
        
        
# Grzes
class RightSpringForceComponent(ReportComponent):

    title="Force in top right spring of the system"

    @property
    def header_text(self):
        #"Energia kinetyczna układu wyrażona jest wzorem:"
        return "Force in the right spring is determined by the formula:"


    @property
    def footer_text(self):
        #"Wyznaczona wielkość określa energię układu wynikającą z jego własności inercyjnych (energię zmagazynowaną w elementach bezwładnych)."
        return "Determined formula specifies force in the spring"

    def append_elements(self):

        system = self.reported_object
        kr=system.k_r
        x_2=system.x_2
        Fr=-kr*x_2
        
        display(ReportText( self.header_text))
        
        display(SympyFormula( Eq(Symbol('F_r'),
                     Fr) , marker=None))

        display(SympyFormula( Eq(Symbol('F_r'),
                     system.right_spring_force()) , marker=None))

        
        display(ReportText( self.footer_text))
        
class CentralSpringForceComponent(ReportComponent):

    title="Force in top centre spring of the system"

    @property
    def header_text(self):
        #"Energia kinetyczna układu wyrażona jest wzorem:"
        return "Force in the top centre spring is determined by the formula:"


    @property
    def footer_text(self):
        #"Wyznaczona wielkość określa energię układu wynikającą z jego własności inercyjnych (energię zmagazynowaną w elementach bezwładnych)."
        return "Determined formula specifies force in the spring"

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        kc=system.k_c
        x_1=system.x_1
        x_2=system.x_2
        Fc=-kc*(x_2-x_1)
        
        display(ReportText( self.header_text))
        
        display(SympyFormula( Eq(Symbol('F_c'),
                     Fc) , marker=None))

        display(SympyFormula( Eq(Symbol('F_c'),
                     system.centre_spring_force()) , marker=None))


        display(ReportText( self.footer_text))
        
class FreeVibrationFrequencyComponent(ReportComponent):
    title = "Natural Frequency"


    @property
    def header_text(self):
        return " natural frequency is given by the formula:"

    @property
    def footer_text(self):
        return "The specific solution is related to the presence of quantities that force motion (vibrations) of the analyzed system."

    def append_elements(self):
        system = self.reported_object
        t = system.ivar
        dyn_sys = system
        dyn_sys_lin = dyn_sys.linearized()

        display(ReportText(self.header_text))

        # Calculate and display the steady solution
        char_poly = dyn_sys_lin.fundamental_matrix().det()
        display(SympyFormula(char_poly))

        omg = Symbol('omega',positive=True)

        # You may want to add more code here to display additional information or perform actions.
#         display(char_poly.coeff(omg**2))
#         display(char_poly.coeff(omg**1))
#         display(char_poly.subs(omg,0))


        display(ReportText(f"Natural Frequency in rad per sec (omage0): "))
#         display(SympyFormula(solve(char_poly,Symbol('omega',positive=True))[0]))
        eigs = dyn_sys_lin.natural_frequencies()

        display(SympyFormula( eigs ))
        
        
        display(SympyFormula( 2*pi*eigs ))
        
        
        display(SympyFormula( (2*pi*eigs)**-1 ))
        
        display(ReportText(self.footer_text))
        
class StaticForceComponent(ReportComponent):

    title="Force of the considered element"

    @property
    def header_text(self):
        return "Force of the considered element is following:"

    def get_force(self):
        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        return dyn_sys.static_force().doit()

    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys

        force=self.get_force()

        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F'),
                         force), marker=None))

class DynamicForceComponent(StaticForceComponent):
    def get_force(self):
        system=self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        return dyn_sys.dynamic_force().doit()
    
class DynamicTensionForceComponent(StaticForceComponent):
    def get_force(self):
        system=self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        return dyn_sys.force_in_cable().doit()

class StaticCableDiameter(ReportComponent):
    
    title="Minimum diameter of the cable due to static force"

    @property
    def header_text(self):
        return "Minimum diameter of the cable due to static force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('d'),
                     dyn_sys.static_cable_diameter().doit() ), marker=None))

class DynamicCableDiameter(ReportComponent):
    
    title="Minimum diameter of the cable due to dynamic force"

    @property
    def header_text(self):
        return "Minimum diameter of the cable due to dynamic force formula:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('d'),
                     dyn_sys.dynamic_cable_diameter().doit() ), marker=None))

class DampedVibrationFrequency(ReportComponent):
    
    title="damped vibration frequency"
    @property
    def header_text(self):
        return "damped vibration frequency(omega'h):"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys


        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('omega_h'),
                     ((list(dyn_sys()._ode_system.general_solution.atoms(sin))[0].args[0])/dyn_sys.ivar)**2 ), marker=None))

class DampedEngineDynamicShearRatioComponent(ReportComponent):
    
    title="Ratio of undamped and damped dynamic shear force"

    @property
    def header_text(self):
        return "Ratio of undamped and damped dynamic shear force is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        from dynpy.models.mechanics.engine import EngineConstantVelocityVerticalSpringGravity
        
        undamped=EngineConstantVelocityVerticalSpringGravity()
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('phi'),
                     undamped.max_static_force()/dyn_sys.max_static_force()), marker=None))

        
class DampingMatrixComponent(ReportComponent):
    title="Damping matrix of the system"
    
    @property
    def header_text(self):
         return "Damping matrix of the system is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('C'),
                     dyn_sys.damping_matrix(), evaluate=False, marker=None)))

class TMDForceComponent(ReportComponent):
    title="Dynamic force in the system with TMD and without"
    
    @property
    def header_text(self):
         return "Dynamic force in the system with TMD and without is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('F_tmd'),
                     dyn_sys.dynamic_force_TMD(), marker=None)))
        display(SympyFormula(Eq(Symbol('F'), dyn_sys.dynamic_force(), marker=None)))
        
class SmallParameterComponent(ReportComponent):
    title="Small parameter"
    
    @property
    def header_text(self):
         return "Small parameter of the system is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('epsilon'),
                     dyn_sys.small_parameter(), marker=None)))
        
class NonlinearSteadySolutionComponent(ReportComponent):
    title="Nonlinear Steady Solution"
    
    @property
    def header_text(self):
         return "Nonlinear steady solution of the system is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(dyn_sys.qs[0], solve(dyn_sys.frequency_response_function(),a)[2]*sin(dyn_sys.Omega*dyn_sys.ivar), marker=None)))
        
class AmplitudeAndFrequencyRelationComponent(ReportComponent):
    title="Nonlinear Steady Solution"
    
    @property
    def header_text(self):
         return "Nonlinear steady solution of the system is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('omega')**2, dyn_sys.freq_amp_rel(), marker=None)))
        
class DynamicTorqueComponent(ReportComponent):
    title="Dynamic Torque"
    
    @property
    def header_text(self):
         return "Dynamic torque of the system is following:"
    
    def append_elements(self):

        system = self.reported_object
        dyn_sys=system
        dyn_sys_lin = dyn_sys
        
        display(ReportText(self.header_text))

        display(SympyFormula( Eq(Symbol('T_d'), dyn_sys.dynamic_torque(), marker=None)))
    
class GivenDataComponent(ReportComponent):

    title="Table with parameter values for calculations"

    @property
    def entry_text(self):

        return "The values of individual parameters adopted for the calculations are presented in the table {tabelkamarker}"
    
    @property
    def table_caption(self):
        return "Default parameter values"


    def append_elements(self):

        dyn_sys = self.reported_object
        
        given_data=dyn_sys._given_data

        if given_data == {}:
            params_dict = dyn_sys.get_reference_data()

        else: params_dict = given_data

        preview_mapper = lambda x: f'${latex(x)}$' if isinstance(x,(Symbol,Eq,Function,Mul)) else str(x)

        mapped_dict = {k: f'${latex(v)}$' if isinstance(v,(Symbol,Eq,Function,Mul)) else str(v) for k, v in params_dict.items()}

        params_dict_for_ldf={'Parameter':list(params_dict.keys()),'Value':list(mapped_dict.values())}
        params_ldf = pd.DataFrame(data=params_dict_for_ldf)
        params_ldf['Parameter'].apply(preview_mapper)
        params_ldf=params_ldf.set_index('Parameter')

        table_params=LatexDataFrame.formatted(params_ldf)

        display(ReportText(self.entry_text.format(tabelkamarker = AutoMarker(table_params))))

        display(table_params.reported(index=True, caption=self.table_caption))

# class SchemeDynPyCodeComponent(ExemplaryPictureComponent):
#     title="Scheme of the system"

#     @property
#     def header_text(self):
#         #"Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."
#         return "Scheme of the real object is presented in the figure. It was obtained by the analysis of real mechanism."


#     @property
#     def footer_text(self):

#         system = self._system

#         #"Analizując przedstawiony układ można stwierdzić, że jego liczba stopni swobody to {len(system.q)}."
#         return f"Analysis of the scheme allows to claim its number of degrees of freedom which is {len(system.q)}"

#     def append_elements(self):

#         system = self._system

#         display(ReportText(  self.header_text ))
#         display(Picture(system._scheme(),width='8cm'))

#         display(ReportText(

# '''The code for calling scheme of the system is following:

#     display(Picture(system._scheme(),width='8cm'))

# '''
#             ))

#         display(ReportText( self.footer_text ))
        
# class SchemeSymPyCodeComponent(ExemplaryPictureComponent):
#     title="Scheme of the system"

#     @property
#     def header_text(self):
#         #"Ilustracja przedstawia schemat rzeczywistego obiektu mechanicznego, wyznaczony na podstawie uprzedniej analizy rzeczywistego obiektu."
#         return "Scheme of the real object is presented in the figure. It was obtained by the analysis of real mechanism."

        
#     @property
#     def footer_text(self):
        
#         system = self._system
        
#         #"Analizując przedstawiony układ można stwierdzić, że jego liczba stopni swobody to {len(system.q)}."
#         return f"Analysis of the scheme allows to claim its number of degrees of freedom which is {len(system.q)}"
    
    
#     def append_elements(self):
        
#         system = self._system
#         path=system._default_folder_path + system.scheme_name

#         display(ReportText(  self.header_text ))
#         display(Picture(path,width='8cm'))
        
#         display(ReportText(
# '''The code for calling scheme of the system is following:

#     path=self._default_folder_path + self.scheme_name
    
#     display(Picture(path ,width='8cm'))

# '''
#             ))
#         display(ReportText( self.footer_text ))
class FouerierExpansionComponent(ReportComponent):
    title='Specific process of Fourier expansion'
    
    @property
    def reported_object(self):
        from dynpy.solvers.linear import ODESystem
        if isinstance(self._reported_object, ODESystem):
            return self._reported_object
        
        elif isinstance(type(self._reported_object), Expr):
                reported_object = self._reported_object
                return reported_object
                
                
        elif isinstance(type(self._reported_object), Eq):
                reported_object = self._reported_object.rhs
                return reported_object
                
        else:
            t=Symbol('t')
            return sin(t)

    @reported_object.setter
    def reported_object(self, obj):
        self._reported_object=obj
        
        
    
    
    
    @property
    def SeriesFormula(self):
        t = Symbol('t')
        kpt = Function('k_p')('t')
        T=Symbol('T')
        n=Symbol('n')
        a0=Symbol('a_0')
        an=Symbol('a_n')
        bn=Symbol('b_n')
        formula=SympyFormula(Eq(kpt,a0/2 + summation(an * cos((n * pi * kpt) / T) + bn * sin((n * pi * kpt) / T), (n, 1, oo))))
        return formula
    
    @property
    def SeriesCoeffs(self):
        t = Symbol('t')
        kpt = Function('k_p')('t')
        T=Symbol('T')
        n=Symbol('n')
        a0=Symbol('a_0')
        an=Symbol('a_n')
        bn=Symbol('b_n')
        coeff1=SympyFormula(Eq(a0,(1/T) * integrate(kpt, (t, -T, T))))
        coeff2=SympyFormula(Eq(an,(2/T) * integrate(kpt * cos((n * pi * t) / T), (t, -T, T))))
        coeff3=SympyFormula(Eq(bn,(2/T) * integrate(kpt * sin((n * pi * t) / T), (t, -T, T))))
        coeffs = [coeff1,coeff2,coeff3]
        return coeffs
    
    @property
    def FFT(self):
        expr=self.reported_object
        fft=expr.to_frequency_domain()
        return fft
        
    

    def append_elements(self):
        
        system = self.reported_object

        display(ReportText("General formula for the Fourier expansion is the following"))
        display(self.SeriesFormula)
        display(ReportText("Formlas for the coefficients of the Fourier series are established in the following manner"))
        for coeff in self.SeriesCoeffs:
            display(coeff)
        



from sympy import (Symbol, symbols, Matrix, sin, cos, asin, diff, sqrt, S,
                   diag, Eq, hessian, Function, flatten, Tuple, im, pi, latex,
                   dsolve, solve, fraction, factorial, Subs, Number, oo, Abs,
                   N, solveset)

from sympy.physics.mechanics import dynamicsymbols, ReferenceFrame, Point
from sympy.physics.vector import vpprint, vlatex
from ...dynamics import LagrangesDynamicSystem, HarmonicOscillator, mech_comp

from ..elements import MaterialPoint, Spring, GravitationalForce, Disk, RigidBody2D, Damper, PID, Excitation, Force, base_frame, base_origin, CombustionEngine
from  ..continuous import ContinuousSystem, PlaneStressProblem
from .elements import Resistor, Capacitor, Inductor

import base64
import random
import IPython as IP
import numpy as np
import inspect


from dynpy.models.mechanics.trolley import ComposedSystem, NonlinearComposedSystem, base_frame, base_origin

from dynpy.models.mechanics.car import CarMovementConstantThrottle


class ElectricCarMovementConstantThrottle(ComposedSystem):

    x = dynamicsymbols('x')
    e = dynamicsymbols('e')

    F_t = Symbol('F_t', positive=True)
    F_p= Symbol('F_p', positive=True)
    F_w = Symbol('F_w', positive=True)
    F_b = Symbol('F_b', positive=True)
    F_n = Symbol('F_n', positive=True)

    T = Symbol('T', positive=True)
    i= Symbol('i', positive=True)
    i_c = Symbol('i_c', positive=True)
    eta = Symbol('eta', positive=True)
    r_d = Symbol('r_d', positive=True)

    m= Symbol('m', positive=True)
    g = Symbol('g', positive=True)
    f_0 = Symbol('f_0', positive=True)
    kappa = Symbol('kappa', positive=True)

    A = Symbol('A', positive=True)
    c_x = Symbol('c_x', positive=True)
    rho = Symbol('rho', positive=True)

    m_red = Symbol('m_red', positive=True)
    J_k= Symbol('J_k', positive=True)
    J_s = Symbol('J_s', positive=True)
    alpha = Symbol('alpha', positive=True)

    a_h = Symbol('a_h', positive=True)
    u = Symbol('u', positive=True)
    
    n = Symbol('n', positive=True)
    
    P = Symbol('P', positive=True)
    I = Symbol('I', positive=True)
    D = Symbol('D', positive=True)
    
    n_min = Symbol('n_min', positive=True)
    n_max = Symbol('n_max', positive=True)
    
    def __init__(self,
                x = None,
                T = None,
                i = None,
                i_c = None,
                eta = None,
                r_d = None,
                m = None,
                g = None,
                f_0 = None,
                kappa = None,
                A = None,
                c_x = None,
                rho = None,
                m_red = None,
                J_k= None,
                J_s = None,
                alpha = None,
                a_h = None,
                u = None,
                n = None,
                P = None,
                I = None,
                D = None,
                e = None,
                n_min = None,
                n_max = None,
                ivar=None,
                **kwargs):

        if x is not None: self.x = x
        if T is not None: self.T = T
        if i is not None: self.i = i
        if eta is not None: self.eta = eta
        if i_c is not None: self.i_c = i_c
        if r_d is not None: self.r_d = r_d
        if m is not None: self.m = m
        if g is not None: self.g = g
        if f_0 is not None: self.f_0 = f_0
        if kappa is not None: self.kappa = kappa
        if A is not None: self.A = A
        if c_x is not None: self.c_x = c_x
        if rho is not None: self.rho = rho
        if m_red is not None: self.m_red = m_red
        if J_k is not None: self.J_k = J_k
        if J_s is not None: self.J_s = J_s
        if alpha is not None: self.alpha = alpha
        if a_h is not None: self.a_h = a_h
        if u is not None: self.u = u
        if n is not None: self.n = n
        if P is not None: self.P = P
        if I is not None: self.I = I
        if D is not None: self.D = D
        if e is not None: self.e = e
        if n_min is not None: self.n_min = n_min
        if n_max is not None: self.n_max = n_max
        if ivar is not None: self.ivar = ivar
            
        self.qs = [self.x]
        
        #self.m_red = (self.m + (4*self.J_k+self.J_s*self.i_c**2)/self.r_d**2)
        
        self.c = S.One/2*self.A*self.c_x*self.rho*diff(self.x,self.ivar)*2/3
        self.mu = self.m_red*self.g*self.f_0*self.kappa
        self.f = self.m_red*self.g*self.f_0
        self.F = (self.T*self.i*self.i_c*self.eta)/self.r_d
        self.n = (diff(self.x,self.ivar)*self.i*self.i_c*60 )/(2*np.pi*self.r_d)
        
        self.error = -7.21914962310842e-24*self.ivar**10 + 4.09285356849447e-20*self.ivar**9 - 9.80641183997517e-17*self.ivar**8 + 1.29051452578393e-13*self.ivar**7 - 1.01472654714565e-10*self.ivar**6 + 4.85741460232082e-8*self.ivar**5 - 1.37892965900947e-5*self.ivar**4 +             0.00213941785802763*self.ivar**3 - 0.148427605950237*self.ivar**2 + 7.13838237925456*self.ivar - 67.8916383683618 - self.x + 67

        self._init_from_components(**kwargs)

        
    @property
    def components(self):

        components = {}

        v = self.x.diff(self.ivar)
        omega = v

        self._car_model = MaterialPoint(self.m_red,
                                     pos1=self.x,
                                     qs=self.qs)(label='car model')

        self._drag = Damper(self.c,
                         pos1=self.x,
                         qs=self.qs)(label='drag')

        self._friction1 = Damper(self.mu,
                         pos1=self.x,
                         qs=self.qs)(label='friction1')

        self._friction2 = Force(-self.f,
                         pos1=self.x,
                         qs=self.qs)(label='friction2')
        
        self._throttle = Force(0.5*self.F+0.5*self.F*sin(3.14*omega/(15)),
                         pos1=self.x,
                         qs=self.qs)(label='throttle')

        components['_car_model'] = self._car_model
        components['_drag'] = self._drag
        components['_friction1'] = self._friction1
        components['_friction2'] = self._friction2
        components['_throttle'] = self._throttle
        return components
    
    @property
    def current_gear(self):
        return 1.024

    def symbols_description(self):
        self.sym_desc_dict = {
            self.T: r'Moment obrotowy',
            self.i: r'Przełożenie obecnego biegu',
            self.eta: r'Sprawność przekładni',
            self.i_c: r'Przełożenie przekładni głównej',
            self.r_d: r'Promień dynamiczny',
            self.m: r'Masa pojazdu',
            self.g: r'Przyśpieszenie ziemskie',
            self.f_0: r'Współczynnik oporu toczenia',
            self.kappa: r'Współczynnik proporcjonalności współczynnika oporu toczenia zależnego od prędkości',
            self.A: r'Powierzchnia czołowa pojazdu',
            self.c_x: r'Współczynnik oporu powietrza',
            self.rho: r'Gęstość powietrza',
            self.m_red: r'Masa redukowana',
            self.J_k: r'Moment bezwładności koła',
            self.J_s: r'Moment bezwładności mas wirujących silnika',
            self.alpha: r'Kąt nachylenia drogi',
            self.a_h: r'Podstawowe opóźnienie siły hamowania',
            self.u: r'Siła hamowania',
            self.x: r'Przemieszczenie w osi x',
            self.ivar: r'Czas',
        }

        return self.sym_desc_dict

    def get_numerical_data(self):

        default_data_dict = {
            self.T: [140],
            self.i: [self.current_gear],
            self.eta: [0.95],
            self.i_c: [4.105],
            self.r_d: [0.97*(0.235*0.45+8.5*0.0254)],
            self.m: [1760],
            self.g: [9.81],
            self.f_0: [0.01],
            self.kappa: [0.00001],
            self.A: [2.14],
            self.c_x: [0.28],
            self.rho: [1.2],
            self.m_red: [1831.6],
            self.J_k: [0.9],
            self.J_s: [0.2],
            self.alpha: [0],
            self.a_h: [4],
            self.P: [210], #2100 # 350
            self.I: [10],
            self.D: [10], #100 # 100
        }

        return default_data_dict
    
    
    def get_default_data(self):

        default_data_dict = {
            self.T: [140],
            self.i: [self.current_gear],
            self.eta: [0.95],
            self.i_c: [4.105],
            self.r_d: [0.97*(0.235*0.45+8.5*0.0254)],
            self.m: [1760],
            self.g: [9.81],
            self.f_0: [0.01],
            self.kappa: [0.00001],
            self.A: [2.14],
            self.c_x: [0.28],
            self.rho: [1.2],
            self.m_red: [1831.6],
            self.J_k: [0.9],
            self.J_s: [0.2],
            self.alpha: [0],
            self.a_h: [4],
            self.P: [210],
            self.I: [0],
            self.D: [100],
        }

        return default_data_dict
    
    
class SecondGearCarMovement(CarMovementConstantThrottle):
    @property
    def current_gear(self):
        return 2.08
    
class ThirdGearCarMovement(CarMovementConstantThrottle):
    @property
    def current_gear(self):
        return 1.361
class FourthGearCarMovement(CarMovementConstantThrottle):
    @property
    def current_gear(self):
        return 0.84

class FifthGearCarMovement(CarMovementConstantThrottle):
    @property
    def current_gear(self):
        return 0.686

    
class OwnCombustionEngine(CombustionEngine):

    eta= Symbol('eta', positive = True)
    i_c = Symbol('i_c', positive = True)
    i = Symbol('i', positive = True)
    r_d= Symbol('r_d', positive = True)
    n_engine = Symbol('n')
    omega=Symbol('omega')

    @property
    def characteristic(self):
        return 1.27125002e-32*self.n_engine**10 -4.80476337e-28*self.n_engine**9  +7.97606590e-24*self.n_engine**8 -7.64635574e-20*self.n_engine**7 +4.67842235e-16*self.n_engine**6 -1.90327141e-12*self.n_engine**5 + 5.18793861e-09*self.n_engine**4 -9.27156707e-06*self.n_engine**3 +1.02140928e-02*self.n_engine**2 -6.02709068e+00*self.n_engine + 1.54285805e+03

class CarMovementAdjustableThrottle(CarMovementConstantThrottle):
    @property
    def current_gear(self):
        return 1.024 # pierwszy bieg
    
    @property
    def components(self):

        components = super().components

        self._throttle = CombustionEngine.from_data(df, n = self.n ,degree=10,qs = self.qs)(label='throttle')
        #self._throttle = CombustionEngine.from_data(df, n = self.n ,degree=10, n_min=1000, n_max=6500,qs = self.qs)(label='throttle')
        #self._throttle = OwnCombustionEngine(omega=self.n,qs=self.qs)(label='throttle')

        components['_throttle'] = self._throttle
        return components
    
class CarMovementRegulatedThrottle(CarMovementConstantThrottle):

    @property
    def components(self):

        components = super().components

        self._throttle = CombustionEngine.from_data(df, n = self.n ,degree=10, qs = self.qs)(label='throttle')
        self._proportional = ProportionalElement(self.P , error=diff(self.x,self.ivar) , target = self.x , reference =30, qs=self.qs)(label='proportional')
        self._integral = IntegralElement(self.I , error=diff(self.x,self.ivar) , target = self.x , reference =30, qs=self.qs)(label='integral')
        self._derivative = DerivativeElement(self.D , error=diff(self.x,self.ivar) , target = self.x, reference =30, qs=self.qs)(label='derivative')

        components['_throttle'] = self._throttle
        components['_proportional'] = self._proportional
        components['_integral'] = self._integral
        components['_derivative'] = self._derivative
        return components
    
class CarMovementRegulatedThrottleACC(CarMovementConstantThrottle):

    @property
    def components(self):

        components = super().components

        self._throttle = CombustionEngine.from_data(df, n = self.n ,degree=10, qs = self.qs)(label='throttle')
        self._proportional = ProportionalElement(self.P , error=self.error, target = self.x, qs=self.qs)(label='proportional')
        self._integral = IntegralElement(self.I , error=self.error, target = self.x, qs=self.qs)(label='integral')
        self._derivative = DerivativeElement(self.D , error=self.error, target = self.x, qs=self.qs)(label='derivative')

        components['_throttle'] = self._throttle
        components['_proportional'] = self._proportional
        #components['_integral'] = self._integral
        components['_derivative'] = self._derivative
        return components
    
    def get_numerical_data(self):

        default_data_dict = {
            self.T: [140],
            self.i: [self.current_gear],
            self.eta: [0.95],
            self.i_c: [4.105],
            self.r_d: [0.97*(0.235*0.45+8.5*0.0254)],
            self.m: [1760],
            self.g: [9.81],
            self.f_0: [0.01],
            self.kappa: [0.00001],
            self.A: [2.14],
            self.c_x: [0.28],
            self.rho: [1.2],
            self.m_red: [1831.6],
            self.J_k: [0.9],
            self.J_s: [0.2],
            self.alpha: [0],
            self.a_h: [4],
            self.P: [5000], #2100
            self.I: [10],
            self.D: [10000], #100
        }

        return default_data_dict
    
class GearboxEngine(ComposedSystem):

    x = dynamicsymbols('x')

    i = Symbol('i', positive=True)
    i_c = Symbol('i_c', positive=True)
    eta = Symbol('eta', positive=True)
    r_d = Symbol('r_d', positive=True)
    n = Symbol('n', positive=True)
    
    data = Symbol('data', positive=True)

    def __init__(self,
                 data = None,
                 i=None,
                 i_c=None,
                 eta=None,
                 r_d=None,
                 n=None,
                 x=None,
                 ivar=None,
                 **kwargs):

        if i is not None: self.i = i
        if i_c is not None: self.i_c = i_c
        if eta is not None: self.eta = eta
        if r_d is not None: self.r_d = r_d
        if n is not None: self.n = n
        if data is not None: self.data = data
        
        if x is not None: self.x = x
        if ivar is not None: self.ivar = ivar
            
        self.qs = [self.x]
        
        self.n_1 = (diff(self.x,self.ivar)*self.i[0]*self.i_c*60 )/(2*np.pi*self.r_d)
        self.n_2 = (diff(self.x,self.ivar)*self.i[1]*self.i_c*60 )/(2*np.pi*self.r_d)
        self.n_3 = (diff(self.x,self.ivar)*self.i[2]*self.i_c*60 )/(2*np.pi*self.r_d)
        self.n_4 = (diff(self.x,self.ivar)*self.i[3]*self.i_c*60 )/(2*np.pi*self.r_d)
        self.n_5 = (diff(self.x,self.ivar)*self.i[4]*self.i_c*60 )/(2*np.pi*self.r_d)
        self.n_6 = (diff(self.x,self.ivar)*self.i[5]*self.i_c*60 )/(2*np.pi*self.r_d)

        self._init_from_components(**kwargs)

    @property
    def components(self):
        components = {}

        self._first_gear =     CombustionEngine.from_data_gearbox(self.data, n = self.n_1 , n_min = 1000 , n_max = 4500 , degree=10 , qs = self.qs)(label='first gear')
        self._second_gear =    CombustionEngine.from_data_gearbox(self.data, n = self.n_2 , n_min = 2565 , n_max = 3800 , degree=10 , qs = self.qs)(label='second gear')
        self._third_gear =     CombustionEngine.from_data_gearbox(self.data, n = self.n_3 , n_min = 2485 , n_max = 3500 , degree=10 , qs = self.qs)(label='third gear')
        self._fourth_gear =    CombustionEngine.from_data_gearbox(self.data, n = self.n_4 , n_min = 2630 , n_max = 3500 , degree=10 , qs = self.qs)(label='fourth gear')
        self._fifth_gear =     CombustionEngine.from_data_gearbox(self.data, n = self.n_5 , n_min = 2835 , n_max = 3500 , degree=10 , qs = self.qs)(label='fifth gear')
        self._sixth_gear =     CombustionEngine.from_data_gearbox(self.data, n = self.n_6 , n_min = 2890 , n_max = 6500 , degree=10 , qs = self.qs)(label='sixth gear')

        components['_first_gear'] = self._first_gear
        components['_second_gear'] = self._second_gear
        components['_third_gear'] = self._third_gear
        components['_fourth_gear'] = self._fourth_gear
        components['_fifth_gear'] = self._fifth_gear
        components['_sixth_gear'] = self._sixth_gear

        return components

    
class CarMovementGearBoxThrottle(CarMovementConstantThrottle):

    @property
    def components(self):

        components = super().components

        self._throttle = GearboxEngine(df, i_gearbox)(label='throttle')
        self._proportional = ProportionalElement(self.P , error=diff(self.x,self.ivar) , target = self.x , reference = 30, qs=self.qs)(label='proportional')
        self._integral = IntegralElement(self.I , error=diff(self.x,self.ivar) , target = self.x , reference = 30, qs=self.qs)(label='integral')
        self._derivative = DerivativeElement(self.D , error=diff(self.x,self.ivar) , target = self.x, reference = 30, qs=self.qs)(label='derivative')

        components['_throttle'] = self._throttle
        components['_proportional'] = self._proportional
        components['_integral'] = self._integral
        components['_derivative'] = self._derivative
        return components
    
class CarMovementPIDAdjust(CarMovementConstantThrottle):

    @property
    def components(self):

        components = super().components

        self._throttle = CombustionEngine.from_data_raw(df, n = self.n ,degree=10,qs = self.qs)(label='throttle')
        self._proportional = ProportionalElement(self.P , error=diff(self.x,self.ivar) , target = self.x , reference = v_ref*self.ivar - 0*self.x*cos(Omega*self.ivar) , qs=self.qs)(label='proportional')
        #self._integral = IntegralElement(self.I , error=diff(self.x,self.ivar) , target = self.x , reference =  v_ref*self.ivar - self.x*cos(Omega*self.ivar) , qs=self.qs)(label='integral')
        self._derivative = DerivativeElement(self.D , error=diff(self.x,self.ivar) , target = self.x, reference =  v_ref*self.ivar - 0*self.x*cos(Omega*self.ivar) , qs=self.qs)(label='derivative')

        components['_throttle'] = self._throttle
        components['_proportional'] = self._proportional
        #components['_integral'] = self._integral
        components['_derivative'] = self._derivative



from sympy import *
from dynpy.solvers.linear import ODESystem
from dynpy.solvers.nonlinear import MultiTimeScaleSolution

class LinearFirstOrder(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z = Function('z')(t)
        a = 10
        ode_eq=z.diff(t)+a*z-sin(10*t)

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=1)

        return odes

class LinearSecondOrder(ODESystem):

    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h= 3^2
        omg0= 14
        fs= 7
        fd= 5
        omg= 2
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z+fd*cos(omg*t)+fs

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes



class LinearHorizontalSystem(ODESystem):

    
    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        z= Function('z')(t)
        h= 3^2
        omg0= 14
        ode_eq=z.diff(t,2)+2*h*z.diff(t)+omg0**2*z

        odes = cls(ode_eq,dvars=z,ivar=t,ode_order=2)

        return odes



#gotowe!!!
class SpringMassSystem(ODESystem):

    
    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        c= 3
        k= 2
        m= 1
        ode_eq=Eq(m*x.diff(t,2)+c*x.diff(t)+k*x,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes
#przyklady FODE
class BasicFODE(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-2*x+1,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEexpExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,exp(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEexpResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,exp(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEsinExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,sin(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEsinResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,sin(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEcosExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x-1,cos(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEcosResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,cos(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEexpSumExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,exp(2*t)+exp(3*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes

class FODEsinSumExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,sin(t)+sin(3*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=1)

        return odes
#PRZYKLADY 2ND ORDER ODE
class BasicSODE(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)+x.diff(t)+x+1,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class BasicSODEWithoutXDot(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)+x+1,0)

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes
    
    
class SODEexpExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,exp(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEexpResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,exp(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEsinExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,sin(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEsinResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,sin(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEcosExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,cos(2*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEcosResonance(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,cos(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEexpSumExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,exp(2*t)+exp(3*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEsinSumExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,sin(t)+sin(3*t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class FODEsincosExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t)-x+1,sin(t)*cos(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes

class SODEsincosExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,sin(t)*cos(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes
    
class SODEsinexpExcitation(ODESystem):


    @classmethod
    def from_reference_data(cls):
        t = Symbol('t')
        x= Function('x')(t)
        ode_eq=Eq(x.diff(t,2)-2*x.diff(t)+x+1,sin(t)*exp(t))

        odes = cls(ode_eq.lhs-ode_eq.rhs,dvars=x,ivar=t,ode_order=2)

        return odes
class SODERectExcitation(ODESystem):
    """
    Class for defining an ODE with rectangular wave excitation.
    """
    @classmethod
    def from_reference_data(cls, period=2*pi, amplitude=1):
        t = Symbol('t')
        x = Function('x')(t)

        # Rectangular wave function
        rect_wave = amplitude * (Heaviside(t % period - period/2) - Heaviside(t % period - period)) 

        # ODE equation
        ode_eq = Eq(x.diff(t, 2) - 2*x.diff(t) + x + 1, rect_wave) 

        odes = cls(ode_eq.lhs - ode_eq.rhs, dvars=x, ivar=t, ode_order=2)

        return odes



class DescriptionHelper:
    
    def summary_param_bank(self):
        '''
        Method contains summarizing descriptions of general parameters.
        '''

        time_domain_summary_param = [
        'Przedstawione wyniki pokazują wpływ zmian badanego parametru \({param_name}\) na dynamikę wózka. ',
        'Opracowane wykresy porównawcze pozwoliły na oszacowanie wpływu zmiennej \({param_name}\) na dynamikę rozpatrywanego układu. ',
        'Zaprezentowane wyniki posłużyły ocenie wrażliwości modelowanego obiektu na badaną zmienną \({param_name}\). ',
        'Przygotowane przebiegi służą zaprezentowaniu wyników symulacji numerycznych badanej zmiennej w funkcji wybranego parametru wpływowego \({param_name}\). ',
        'Analizowany model wykazuje wrażliwość na zmianę rozważanego parametru \({param_name}\). ',
        ]
        time_domain_summary_x = [
        'Zaobserwowano wpływ badanego parametru \({x(t)}\) na drgania wózka, których maksymalna wartość amplitudy wyniosła {x(t)_max}. ',
        'Analizowany przebieg parametru \({x(t)}\) osiąga wartość maksymalną równą {x(t)_max}. ',
        'Minimalna wartość dla przebiegu \({x(t)}\) jest równa {x(t)_min}. ',
        'Na podstawie przeprowadzonych badań numerycznych można stwierdzić, że w analizowanym przebiegu parametru \({x(t)}\) nie występują amplitudy o wartości mniejszej niż {x(t)_min} oraz  większej niż {x(t)_max}. ',
        'Dokonując analizy wyników symulacji dla parametru \({x(t)}\) stwierdza się, że amplitudy drgań nie przekraczają wartości {x(t)_max}, a wartość mininalna wynosi {x(t)_min}. ',
        ]

        return self
        
    def summary_var_bank(self):
        '''
        Method include summarizing descriptions of generalized variables.
        '''
        
        time_domain_summary_z = [ 
        'Podsumowując symulację możemy określić, że wartość maksymalna dla współrzędnej uogólnionej \({z(t)}\) jest równa {z(t)_max}. ',
        'Poddawane analizie sygnały cechują się wartością maksymalną równą {z(t)_max} dla współrzędnej \({z(t)}\). ',
        'Przebieg współrzędnej \({z(t)}\) charakteryzuje się wartością minimalną równą {z(t)_min}. ',
        'Wykonane badania numeryczne umożliwiają stwierdzenie, że odpowiedź czasowa \({z(t)}\) nie przekracza {z(t)_max} i jest ograniczana z dołu przez {z(t)_min}. ',
        'Opierając się na analizie przeprowadzonych badań stwierdzono, że współrzędna \({z(t)}\) nie przekracza wartości {z(t)_max}, a jej wartość mininalna to {z(t)_min}. ',
        ]
        time_domain_summary_z_wrc = [ 
        'Przedłożone wyniki symulacji pokazują, że wartość maksymalna dla współrzędnej uogólnionej \({z_wrc(t)}\) wynosi {z_wrc(t)_max}. ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_wrc(t)_max} dla współrzędnej \({z_wrc(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_wrc(t)}\) jest równa {z_wrc(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({z_wrc(t)}\) nie przekracza {z_wrc(t)_max} i jest ograniczona z dołu przez {z_wrc(t)_min}. ',
        'Po dokonaniu analizy badań zaobserwowano, że współrzędna \({z_wrc(t)}\) nie przekracza wartości {z_wrc(t)_max}, a wartość mininalna wynosi {z_wrc(t)_min}. ',
        ]
        time_domain_summary_z_r = [ 
        'Wyniki symulacji ukazują, że wartość maksymalna dla współrzędnej uogólnionej \({z_r(t)}\) wynosi {z_r(t)_max}. ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_r(t)_max} dla współrzędnej \({z_r(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_r(t)}\) jest równa {z_r(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({z_r(t)}\) nie przekracza {z_r(t)_max} i jest ograniczona z dołu przez {z_r(t)_min}. ',
        'Na podstawie analizy przeprowadzonych badań stwierdzono, że współrzędna \({z_r(t)}\) nie przekracza wartości {z_r(t)_max}, a wartość mininalna wynosi {z_r(t)_min}. ',
        ]
        time_domain_summary_z_fr = [ 
        'Rezultaty symulacji pokazują wartość maksymalną {z_fr(t)_max} dla współrzędnej uogólnionej \({z_fr(t)}\). ',
        'Analizowane sygnały reprezentujące odpowiedż czasową układu charakteryzują się wartością maksymalną równą {z_fr(t)_max} dla współrzędnej \({z_fr(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({z_fr(t)}\) jest równa {z_fr(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają określić, że odpowiedź czasowa \({z_fr(t)}\) nie przekracza {z_fr(t)_max} i jest ograniczana z dołu przez {z_fr(t)_min}. ',
        'Poprzez analizie przeprowadzonych badań zaobserwowano, że współrzędna \({z_fr(t)}\) nie przekracza wartości {z_fr(t)_max}, a wartość mininalna wynosi {z_fr(t)_min}. ',
        ]
        
        return self
    
    def summary_angle_bank(self):
        '''
        Method includes summarizing descriptions of examined angles.
        '''
        time_domain_summary_varphi = [ 
        'Wyniki symulacji prezentują wartość maksymalną {varphi(t)_max} dla współrzędnej uogólnionej \({varphi(t)}\). ',
        'Analizowane sygnały reprezentujące informacje zwrotną czasową układu charakteryzują się wartością maksymalną równą {varphi(t)_max} dla współrzędnej \({varphi(t)}\). ',
        'Wartość minimalna przebiegu współrzędnej \({varphi(t)}\) wynosi {varphi(t)_min}. ',
        'Przeprowadzone badania numeryczne pozwalają stwierdzić, że odpowiedź czasowa \({varphi(t)}\) nie przekracza {varphi(t)_max} i jest ograniczona z dołu przez {varphi(t)_min}. ',
        'Analiza przeprowadzonych badań umożliwiła wyciągnięcie wniosków, że w przypadku współrzędnej \({varphi(t)}\) nie dochodzi do przekroczenia wartości {varphi(t)_max}, a wartość     mininalna wynosi {varphi(t)_min}. ',
        ]
        time_domain_summary_theta = [ 
        'Rezultaty symulacji wykazują, że wartość maksymalna dla współrzędnej uogólnionej \({theta(t)}\) osiąga wartość {theta(t)_max}. ',
        'Weryfikowane sygnały reprezentujące rezultat czasowy układu wyróżniają się wartością maksymalną równą {theta(t)_max} dla współrzędnej \({theta(t)}\). ',
        'Przebieg współrzędnej \({theta(t)}\) cechuje się wartością minimalną {theta(t)_min}. ',
        'Badania numeryczne pozwalają wywnioskować, że informacja zwrotna czasowa \({theta(t)}\) nie przekracza {theta(t)_max} oraz jest ograniczona z dołu przez {theta(t)_min}. ',
        'Biorąc pod uwagę przeprowadznone badania można zaobserwować, że współrzędna \({theta(t)}\) nie przekracza wartości {theta(t)_max}, a wartość mininalna wynosi {theta(t)_min}. ',
        ]
        time_domain_summary_varphi_RC = [ 
        'Wyniki symulacji pokazują, że wartość maksymalna dla współrzędnej uogólnionej \({varphi_RC(t)}\) jest równa {varphi_RC(t)_max}. ',
        'Analizowane sygnały charakteryzują się wartością maksymalną równą {varphi_RC(t)_max} dla współrzędnej \({varphi_RC(t)}\).',
        'Wartość minimalna przebiegu współrzędnej \({varphi_RC(t)}\) jest określona wartością {varphi_RC(t)_min}. ',
        'Odpowiedź czasowa \({varphi_RC(t)}\) nie przekracza {varphi_RC(t)_max} i jest ograniczana z dołu przez {varphi_RC(t)_min}, co pozwala nam stwierdzić analiza przeprowadzonych badań. ',
        'Bazując na analizie przeprowadzonych badań zaobserwowano, że współrzędna \({varphi_RC(t)}\) nie przekracza wartości {varphi_RC(t)_max}, a wartość mininalna wynosi {varphi_RC(t)_min}. ',
        ]
        
        return self
    
    def general_summary(self):
        '''
        Method includes general concluding sentences.
        '''
        # to improve
        time_domain_summary_tval = [
        'Zaobserwowane odczyty wynikające z wpływu \({param_name}\) występują w konketrnym czasie ruchu {t_val}. ',
        'Odczytana maksymalna wartość amplitudy {max_val} wynika z działania \({param_name}\) wystąpiła konkretnie w czasie {t_val}. '
        'Minimalny zaobserwowany wynik amplitudy {min_val} miał miejsce w {t_val} czasu trwania pomiaru. ',
        ]


        freq_domain_summary =[
        'Ponadto sprawdzono własności spektralne otrzymanych wyników symulacyjnych.',
        'Według przeprowadzonych badań modelowych struktura częstotliwościowa nie zawiera komonentów większych niż {max_val_spec}.',
        'Amplitudy drgań badanego parametru \({q_name}\) przyjmują maksymalną wartość równą {max_val_spec}.',
        'Na podstawie analizy widmowej minimalna wartość częstotliwości rozpatrywanego parametru wynosi {min_val_spec}.',


        ]

        summary_bank_optimization=[
        'Zaprezentowano efekty symulacji optymalizacyjnych dla badanego modelu dynamicznego wózka inwalidzkiego wraz z zamontowanym napędem RC. Dla uzyskanych danych symulacyjnych przygotowano wykresy przedstawiające funkcję celu w funkcji analizowanego parametru. Analizowane dane symulacyjne pozwalają na określenie wartości minimalnej'
        ]
        
        return self
    
    def sec_introduction_gen(self):
        '''
        Method contains sentences describing process of research.
        '''
        sec_introduction_bank_gen1=[
        'Wykorzystując przygotowany model dynamiczny, przeprowadzono serię symulacji mających na celu ocenę wrażliwości modelu na zmiany wybranych parametrów. ',
        'Przeprowadzenie analizy numerycznej wymaga znajomości modelu dynamicznego oraz zdeterminowania parametrów oddziałujących na zachowanie układu wskutek ich zmian. ',
        'Wykorzystując przygotowane środowisko obliczeniowe, wykonano szereg symulacji, opracowano dane numeryczne i przedstawiono je w postaci następujących wykresów. ',
        'Celem zbadania i oceny dynamiki rozpatrywanego systemu przeprowadzono symulacje numeryczne przedstawijące przebiegi zmiennych układu w funkcji czasu. '
        'W oparciu o wybrany model dynamiczny wózka, wykonano analizy numeryczne zaprezentowane na kolejnych wykresach. '
        ]

        sec_introduction_bank_gen2=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu możliwe będzie wprowadzenie zmian, które wpłyną na poprawę działania modelu. ',
        'Charakter zmian przebiegów poszczególnch współrzędnych może posłużyć do opracowania wniosków, na podstawie których możliwe będzie ulepszenie funkcjonowania rozważanego układu. ',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu maja na celu ułatwienie analizy zachowania modelu dynamicznego oraz wprowadzenie ewentualnych poprawek na podstawie dokonanych obserwacji i spostrzeżeń. ',
        'Dostrzegając wzajemne zależności między dynamicznymi parametrami wózka możliwe będzie wprowadzenie takich zmian, które w korzystny sposób wpłyną na odpowiedź układu. '
        ]


        sec_introduction_bank_gen3=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu możliwe będzie wprowadzenie zmian, które wpłyną na poprawę działania modelu. ',
        'Charakter zmian przebiegów poszczególnch współrzędnych może posłużyć do opracowania wniosków, na podstawie których możliwe będzie ulepszenie funkcjonowania rozważanego układu. ',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu maja na celu ułatwienie analizy zachowania modelu dynamicznego oraz wprowadzenie ewentualnych poprawek na podstawie dokonanych obserwacji i spostrzeżeń. ',
        'Dostrzegając wzajemne zależności między dynamicznymi parametrami wózka możliwe będzie wprowadzenie takich zmian, które w korzystny sposób wpłyną na odpowiedź układu. '
        ]
        
        return self
    
    def introduction_gen(self):
        '''
        Method includes sentences describing received data and graphs.
        '''
        
        introduction_bank_fig_no=[
        'Na wykresie {nr_rys} przedstawiano zmiany wielkości dynamicznych charakteryzujących ruch wózka. Po przeanalizowaniu można zanotować wzajemną zależność poszczególnych wielkości dynamicznych.',
        'Charakter przebiegu wartości dynamicznych wózka został przedstawiony na rysunku {nr_rys}.',
        'Wykres {nr_rys} pokazuje zmienność parametrów dynamicznych wózka w trakcie symulownego przejazdu.',
        'Rysunek {nr_rys} prezentuje wykres opisujący zmienność parametrów dynamicznych wózka w trakcie jazdy po zamodelowanym torze.',
        ]

        introduction_bank_given_data=[
        'Symulacja została przeprowadzona dla następujących danych: {given_data}.',
        'Zaprezentowane wyniki numeryczne otrzymano dla danych równych:{given_data}.',
        'Eksperyment numeryczny został przeprowadzona dla następujących danych:{given_data}.',
        'Wyniki stmulacji numerycznych otrzymano dla:{given_data}.',
        ]
        introduction_bank_gen1=[
        'Na podstawie spostrzeżeń wynikających z obserwacji odpowiedzi czasowych analizowanego układu stwierdza się oscylacyjny charakter odpowiedzi układu. Dynamika systemu ma stabilny charakter',
        'Charakter zmian przebiegów poszczególnch współrzędnych jest stabilny. Otrzymane sygnały mieszczą się w zakresie dopuszczalnych uwarunkowaniami fizycznymi.',
        'Zaprezentowane wykresy przedstawiające odpowiedzi czasowe układu mają drganiowy charakter i są zgodne z oczekiwaniami.',
        'Bazując na otrzymanych wynikach można stwierdzić wzajemną zależność poszczególnych wielkości dynamicznych. Dodatkow wielkości modelu stabilizują się w czasie. ',
        ]
        
        return self
    
    def conclusion_param_bank(self):
        '''
        Method contains parameters' concluding descriptions.
        '''
        conclusion_bank_x=['Zauważa się {x(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({x(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla {x(t)_idmax} i {Derivative(x(t), t)_idmax}. ',
                           
        'Zaobserwowano {x(t)_inf} parametru \({param_name}\) na maksymalną wartość pokonanej drogi oraz osiąganą wartość prędkości. Maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla maksymalnej wartości badanego parametru równej {x(t)_idmax}. ',
                           
        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({x(t)}\), gdzie największa wartość pokonanej drogi to {x(t)_max}. W konsekwencji zaobserwowano {x(t)_inf} analizowanej zmiennej na wartość prędkości liniowej \({Derivative(x(t), t)}\), dla której minimalna wartość wynosi {Derivative(x(t), t)_min}, a największą osiąganą wartością jest {Derivative(x(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(x(t), t)_idmin} oraz {Derivative(x(t), t)_idmax}. ',
                  ]
        
        return self
    
    def conclusion_var_bank(self):
        '''
        Method contains variables' concluding descriptions.
        '''
        conclusion_bank_z=['Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({z(t)}\), gdzie największa wartość amplitudy to {z(t)_max}, a najmniejsza {z(t)_min}. W konsekwencji zaobserwowano {z(t)_inf} analizowanej zmiennej na wartość drgań prędkości drgań pionowych, dla których minimalna wartość amplitudy to {Derivative(z(t), t)_min}, a największą osiąganą wartością jest {Derivative(z(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(z(t), t)_idmin} oraz {Derivative(z(t), t)_idmax}. ',

        'Zauważa się {z(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({z(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {z(t)_max} oraz {Derivative(z(t), t)_max} dla {z(t)_idmax} i {Derivative(z(t), t)_idmax}. '

        'Zaobserwowano {z(t)_inf} parametru \({param_name}\) na maksymalną wartość drgań pionowych wózka oraz osiąganą wartość ich prędkości. Maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {z(t)_max} oraz {Derivative(z(t), t)_max} dla maksymalnej wartości badanego parametru równej {z(t)_idmax}. '
                ]
        
        return self
    
    def conclusion_angle_bank(self):
        '''
        Method contains angles' concluding descriptions.
        '''
        
        
        conclusion_bank_varphi_rc=['Zaobserwowano {varphi_RC(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max} odpowiednio dla wartości parametru: {varphi_RC(t)_idmin} oraz {varphi_RC(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}.' ,

        'Zmiana \({param_name}\) ma {varphi_RC(t)_inf} na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max} odpowiednio dla wartości parametru: {varphi_RC(t)_idmin} oraz {varphi_RC(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. ',

        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({varphi_RC(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi_RC(t)_min}. W konsekwencji zaobserwowano {varphi_RC(t)_inf} analizowanej zmiennej na wartość prędkości kątową napędu RC. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(varphi_RC(t), t)_idmin} oraz {Derivative(varphi_RC(t), t)_idmax}. ' ,
                ]

        conclusion_bank_varphi_zwrc=['Zaobserwowano {z_wrc(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań pionowych i prędkość napędu RC. Przemieszczenia pionowe nie przyjmują wartości mniejszej niż {z_wrc(t)_min} oraz większej niż {z_wrc(t)_max} odpowiednio dla wartości parametru: {z_wrc(t)_idmin} oraz {z_wrc(t)_idmax}. Dla prędkości drgań pionowych napędu minimalna wartość amplitudy to {Derivative(z_wrc(t), t)_min}, a największą osiąganą wartością jest {Derivative(z_wrc(t), t)_max}. ',

        'Zmiana \({param_name}\) ma {z_wrc(t)_inf} na wartość drgań i prędkość kątową analizowanego układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {z_wrc(t)_min} oraz większej niż {z_wrc(t)_max} odpowiednio dla wartości parametru: {z_wrc(z_wrct)_idmin} oraz {z_wrc(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(z_wrc(t), t)_min}, a największą osiąganą wartością jest {Derivative(z_wrc(t), t)_min}. ',

        'Zmianę dynamiki pod wpływem zmienności parametru \({param_name}\) obserwuje się dla \({varphi(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi(t)_min}. W konsekwencji zaobserwowano {varphi(t)_inf} analizowanej zmiennej na wartość prędkości drgań kątowych badanego układu, dla którego minimalna wartość amplitudy wynosi {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_max} odpowiednio dla wartości parmametru: {Derivative(varphi(t), t)_idmin} oraz {Derivative(varphi(t), t)_idmax}. '
                ]

        conclusion_bank_phi=['Zmiana \({param_name}\) ma {varphi(t)_inf} na wartość drgań i prędkość kątową układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi(t)_min} oraz większej niż {varphi(t)_max} odpowiednio dla wartości parametru: {varphi(t)_idmin} oraz {varphi(t)_idmax}. Dla prędkości kątowej wózka minimalna wartość amplitudy to {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_min}. ',
    
        'Zaobserwowano {varphi(t)_inf} rozpatrywanego parametru - \({param_name}\) na wartość drgań i prędkość kątową układu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi(t)_min} oraz większej niż {varphi(t)_max} odpowiednio dla wartości parametru: {varphi(t)_idmin} oraz {varphi(t)_idmax}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi(t), t)_max}. ',

        'Zauważa się {varphi(t)_inf} zmienności parametru \({param_name}\) dla współrzędnej \({varphi(t)}\) oraz odpowiadającej temu przemieszczniu kątowemu prędkości. Stwierdzono wpływ badanego parametru, gdzie maksymalne wartości dla wymienionych współrzędnych przyjmują odpowiednio {varphi(t)_max} oraz {Derivative(varphi(t), t)_max} dla {varphi(t)_idmax} i {Derivative(varphi(t), t)_idmax}. ',
                ]
        
        return self
    
    def no_impact_conclusion_bank(self):
        '''
        Method contains sentences describing no impact on any different variables.
        '''
        conclusion_bank_no_impact=['Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. ',
                           'Nie zauważono kluczowych zmian pozostałych zmiennych układu pod wpływem badanych wartości rozpatrywanego parametru. ',
                           'Pozostałe zmienne układu nie wykazują wpływu oddziałującego w sposób istoty na dynamikę rozpatrywanego systemu. ',
                           'Badany system dynamiczny nie wykazuje szczególnej wrażliwości na zmiany innych analizowanych parametrów. ',
                ]
        
        return self
    
    def sensitivity_anal_dict(self):
        '''
        Method contains dictionary with described parameters connected with sensitivity.
        '''
        sensitivity_anal_dict = {
                        k_fix:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu.',
                        m_RC:'Zmiana masy napędy RC w sposób zauważalny wpłynęła na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. Mniej istotne zmiany zaobserwowano dla parametru \({x(t)}\) oraz odpowiadającej temu przemieszczniu prędkości. Nie zauważono kluczowych zmian pozostałych zmiennych układu. ',
                        F:'Zaobserwowano istotny wpływ tego parametru na maksymalną wartość pokonanej drogi oraz osiąganą wartość prędkości, gdzie zaobserwować można liniową zależność. Maksymalne wartośći dla wymienionych współrzędnych przyjmują odpowiednio {x(t)_max} oraz {Derivative(x(t), t)_max} dla maksymalnej wartości siły. Zmianę dynamiki obserwuje się również dla \({varphi_RC(t)}\), gdzie największa wartość amplitudy to {varphi_RC(t)_max}, a najmniejsza {varphi_RC(t)_min}. ',
                        l_RC:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. ',
                        m_3:'Zmiana masy wózka w sposób zauważalny wpłynęła na wartość drgań i prędkość kątową napędu. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_min}. Istotme zmiany zaobserwowano również dla parametru \({x(t)}\) - pokonanej drogi oraz \({Derivative(x(t), t)}\) - prędkości, gdzie zależność przedstawia się funkcją malejąca dla coraz większej masy wózka. Nie zauważono kluczowych zmian pozostałych zmiennych układu. ',
                        k_r:'Zaobserwowano wpływ rozpatrywanego parametru na wartość drgań i prędkość kątową napędu RC. Przemieszczenia kątowe nie przyjmują wartości mniejszej niż {varphi_RC(t)_min} oraz większej niż {varphi_RC(t)_max}. Dla prędkości kątowej napędu minimalna wartość amplitudy to {Derivative(varphi_RC(t), t)_min}, a największą osiąganą wartością jest {Derivative(varphi_RC(t), t)_max}. Nie odmotowano natomiast istotnego wpływu tego parametru na pozostałe stopnie swobody układu. '
                        }
        
        return self





summary_dict={time_param:time_domain_summary_param,
              time_x:time_domain_summary_x,
              time_varphi:time_domain_summary_varphi
              time_z:time_domain_summary_z,
              time_theta:time_domain_summary_theta,
              time_z_wrc:time_domain_summary_z_wrc,
              time_z_r:time_domain_summary_z_r,
              time_z_fr:time_domain_summary_z_fr,
              time_varphi_RC:time_domain_summary_varphi_RC,
              time_tval:time_domain_summary_tval,
              freq:freq_domain_summary,
              opti:summary_bank_optimization
             }


summary_bank_composed_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_x,
        time_domain_summary_varphi,
        time_domain_summary_z,
        time_domain_summary_varphi_RC,
            )) for obj in range(30)  ]    

summary_bank_chair_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_x,
        time_domain_summary_varphi,
        time_domain_summary_z,
            )) for obj in range(30)  ]   


summary_bank_drive_model=[str(RandomDescription(
        time_domain_summary_param,
        time_domain_summary_varphi_RC,
            )) for obj in range(30)  ]      


introduction_dict={gen1:sec_introduction_bank_gen1,
                  gen2:sec_introduction_bank_gen2,
                  gen3:sec_introduction_bank_gen3
                  }

sec_intro_composed_model=[str(RandomDescription(sec_introduction_bank_gen1,sec_introduction_bank_gen2,sec_introduction_bank_gen3)) for obj in range(30)  ]  


refs_dict={fig_no:introduction_bank_fig_no,
          given_data:introduction_bank_given_data}

intro_bank_composed_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]
intro_bank_chair_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]
intro_bank_drive_model=[str(RandomDescription(introduction_bank_fig_no,introduction_bank_given_data,introduction_bank_gen1)) for obj in range(30)  ]


conclusion_bank_composed_model=[str(RandomDescription( conclusion_bank_varphi_rc,conclusion_bank_x ,conclusion_bank_z,conclusion_bank_phi,conclusion_bank_no_impact)) for obj in range(30)  ]
conclusion_bank_chair_model=[str(RandomDescription(conclusion_bank_x,conclusion_bank_z,conclusion_bank_phi,conclusion_bank_no_impact)) for obj in range(30)  ]
conclusion_bank_drive_model=[str(RandomDescription(conclusion_bank_varphi_rc,conclusion_bank_no_impact)) for obj in range(30)  ]
